@@ -1,15 +1,21 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2021, Oracle and/or its affiliates.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
-# License as published by the Free Software Foundation; version 2
-# of the License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
+#
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
@@ -31,9 +37,7 @@ use File::Basename;
 #
 # Rules to run first of all
 #
-my @pre_rules=
-(
-);
+my @pre_rules = (\&pre_check_no_hosts_in_cluster_config);
 
 
 my @share_locations= ("share/mysql", "sql/share", "share");
@@ -53,6 +57,15 @@ sub get_testdir {
   return $testdir;
 }
 
+# Retrive build directory (which is different from basedir in out-of-source build)
+sub get_bindir {
+  if (defined $ENV{MTR_BINDIR})
+  {
+    return $ENV{MTR_BINDIR};
+  }
+  my ($self, $group)= @_;
+  return $self->get_basedir($group);
+}
 
 sub fix_charset_dir {
   my ($self, $config, $group_name, $group)= @_;
@@ -62,8 +75,8 @@ sub fix_charset_dir {
 
 sub fix_language {
   my ($self, $config, $group_name, $group)= @_;
-  return my_find_dir($self->get_basedir($group),
-		     \@share_locations, "english");
+  return my_find_dir($self->get_bindir($group),
+		     \@share_locations);
 }
 
 sub fix_datadir {
@@ -84,6 +97,11 @@ sub fix_port {
   return $self->{HOSTS}->{$hostname}++;
 }
 
+sub fix_x_port {
+  my ($self, $config, $group_name, $group)= @_;
+  return $self->{ARGS}->{mysqlxbaseport}++;
+}
+
 sub fix_host {
   my ($self)= @_;
   # Get next host from HOSTS array
@@ -125,11 +143,29 @@ sub fix_server_id {
   return $server_id;
 }
 
+sub fix_x_socket {
+  my $result = fix_socket(@_);
+  $result =~ s/mysqld\.([0-9]+)\.sock$/mysqlx\.$1\.sock/;
+
+  return $result;
+}
+
 sub fix_socket {
   my ($self, $config, $group_name, $group)= @_;
   # Put socket file in tmpdir
   my $dir= $self->{ARGS}->{tmpdir};
-  return "$dir/$group_name.sock";
+  my $socket = "$dir/$group_name.sock";
+ 
+  # Make sure the socket path does not become longer then the path
+  # which mtr uses to test if a new tmpdir should be created
+  if (length($socket) > length("$dir/mysql_testsocket.sock"))
+  {
+    # Too long socket path, generate shorter based on port
+    my $port = $group->value('port');
+    $socket = "$dir/mysqld-$port.sock"; 
+  }
+
+  return $socket;
 }
 
 sub fix_tmpdir {
@@ -141,7 +177,11 @@ sub fix_tmpdir {
 sub fix_log_error {
   my ($self, $config, $group_name, $group)= @_;
   my $dir= $self->{ARGS}->{vardir};
-  return "$dir/log/$group_name.err";
+  if ( $::opt_valgrind and $::opt_debug ) {
+    return "$dir/log/$group_name.trace";
+  } else {
+    return "$dir/log/$group_name.err";
+  }
 }
 
 sub fix_log {
@@ -174,11 +214,12 @@ sub ssl_supported {
   return $self->{ARGS}->{ssl};
 }
 
-sub fix_skip_ssl {
+sub fix_ssl_disabled {
   return if !ssl_supported(@_);
-  # Add skip-ssl if ssl is supported to avoid
+  return if $::opt_ssl;
+  # Add ssl-mode=DISABLED to avoid
   # that mysqltest connects with SSL by default
-  return 1;
+  return "DISABLED";
 }
 
 sub fix_ssl_ca {
@@ -187,6 +228,12 @@ sub fix_ssl_ca {
   return "$std_data/cacert.pem"
 }
 
+sub fix_client_ssl_ca {
+  return if !$::opt_ssl;
+  my $std_data= fix_std_data(@_); 
+  return "$std_data/cacert.pem"
+}
+
 sub fix_ssl_server_cert {
   return if !ssl_supported(@_);
   my $std_data= fix_std_data(@_);
@@ -221,12 +268,14 @@ my @mysqld_rules=
  { 'basedir' => sub { return shift->{ARGS}->{basedir}; } },
  { 'tmpdir' => \&fix_tmpdir },
  { 'character-sets-dir' => \&fix_charset_dir },
- { 'language' => \&fix_language },
+ { 'lc-messages-dir' => \&fix_language },
  { 'datadir' => \&fix_datadir },
  { 'pid-file' => \&fix_pidfile },
  { '#host' => \&fix_host },
  { 'port' => \&fix_port },
  { 'socket' => \&fix_socket },
+ { 'loose-mysqlx-port' => \&fix_x_port },
+ { 'loose-mysqlx-socket' => \&fix_x_socket },
  { '#log-error' => \&fix_log_error },
  { 'general_log' => 1 },
  { 'general_log_file' => \&fix_log },
@@ -240,8 +289,10 @@ my @mysqld_rules=
  { 'ssl-ca' => \&fix_ssl_ca },
  { 'ssl-cert' => \&fix_ssl_server_cert },
  { 'ssl-key' => \&fix_ssl_server_key },
+ { 'loose-sha256_password_auto_generate_rsa_keys' => "0"},
   );
 
+
 if (IS_WINDOWS)
 {
   # For simplicity, we use the same names for shared memory and 
@@ -272,20 +323,6 @@ sub fix_cluster_backup_dir {
 }
 
 
-sub fix_cluster_undo_file_dir {
-  my ($self, $config, $group_name, $group)= @_;
-  my $dir= $group->value('DataDir');
-  return "$dir/uf";
-}
-
-
-sub fix_cluster_data_file_dir {
-  my ($self, $config, $group_name, $group)= @_;
-  my $dir= $group->value('DataDir');
-  return "$dir/df";
-}
-
-
 #
 # Rules to run for each ndb_mgmd in the config
 #  - will be run in order listed here
@@ -306,11 +343,19 @@ my @ndbd_rules=
  { 'HostName' => \&fix_host },
  { 'DataDir' => \&fix_cluster_dir },
  { 'BackupDataDir' => \&fix_cluster_backup_dir },
- { 'FileSystemPathDD' => \&fix_cluster_undo_file_dir },
- { 'FileSystemPathDataFiles' => \&fix_cluster_data_file_dir },
 );
 
 
+#
+# Rules to run for each memcached in the config
+#  - will be run in order listed here
+#
+my @memcached_rules=
+(
+ { '#host' => \&fix_host },
+ { 'port' => \&fix_port },
+);
+
 #
 # Rules to run for each cluster_config section
 #  - will be run in order listed here
@@ -339,10 +384,10 @@ my @client_rules=
 #
 my @mysqltest_rules=
 (
- { 'ssl-ca' => \&fix_ssl_ca },
+ { 'ssl-ca' => \&fix_client_ssl_ca },
  { 'ssl-cert' => \&fix_ssl_client_cert },
  { 'ssl-key' => \&fix_ssl_client_key },
- { 'skip-ssl' => \&fix_skip_ssl },
+ { 'ssl-mode' => \&fix_ssl_disabled },
 );
 
 
@@ -353,6 +398,7 @@ my @mysqltest_rules=
 my @mysqlbinlog_rules=
 (
  { 'character-sets-dir' => \&fix_charset_dir },
+ { 'local-load' => sub { return shift->{ARGS}->{tmpdir}; } },
 );
 
 
@@ -362,10 +408,48 @@ my @mysqlbinlog_rules=
 #
 my @mysql_upgrade_rules=
 (
- { 'tmpdir' => sub { return shift->{ARGS}->{tmpdir}; } },
 );
 
 
+# For [cluster_config] section there should be no ndbd, ndb_mgmd, mysqld,
+# ndbapi options set.  These should only be set in cluster instance specific
+# sections, [cluster_config.X].  This is crucial when determining in the order
+# ndb_mgmd will enumerate node ids for nodes without explicit node id.
+sub pre_check_no_hosts_in_cluster_config {
+  my ($self, $config) = @_;
+
+  my $group = $config->group('cluster_config');
+  if (defined $group) {
+    if (defined $group->if_exist('ndbd') ||
+        defined $group->if_exist('ndb_mgmd') ||
+        defined $group->if_exist('mysqld') ||
+        defined $group->if_exist('ndbapi')) {
+      croak "Configuration error: Do not set ndbd, ndb_mgmd, mysqld, ndbapi ".
+            "in [cluster_config] section.  Use cluster specific sections ".
+            "[cluster_config.X] instead.";
+    }
+  }
+}
+
+# For each [cluster_config.X] record all allocated node ids (set using NodeId
+# property).  This will be used when determining what node id nodes with no
+# configured node id will get.
+sub track_allocated_nodeid {
+  my ($self, $config, $group_name, $group) = @_;
+
+  my (undef, $process_type, $idx, $suffix) = split(/\./, $group_name);
+
+  if (my $nodeid = $group->if_exist('NodeId')) {
+    if (!defined $suffix) {
+      croak "Configuration error: Do not set NodeId in general group ".
+            "[$group_name] but use instance specific group like ".
+            "[cluster_config.$process_type.1.cluster] instead.";
+    }
+    $config->{"cluster_config.$suffix"}->{"ALLOCATED NODEIDS"}->{$nodeid} = 1;
+  }
+  return;
+}
+
 #
 # Generate a [client.<suffix>] group to be
 # used for connecting to [mysqld.<suffix>]
@@ -373,6 +457,7 @@ my @mysql_upgrade_rules=
 sub post_check_client_group {
   my ($self, $config, $client_group_name, $mysqld_group_name)= @_;
 
+
   #  Settings needed for client, copied from its "mysqld"
   my %client_needs=
     (
@@ -382,7 +467,6 @@ sub post_check_client_group {
      user       => '#user',
      password   => '#password',
     );
-
   my $group_to_copy_from= $config->group($mysqld_group_name);
   while (my ($name_to, $name_from)= each( %client_needs )) {
     my $option= $group_to_copy_from->option($name_from);
@@ -474,12 +558,21 @@ sub resolve_at_variable {
   my $group_name=  join('.', @parts);
 
   $group_name =~ s/^\@//; # Remove at
-
-  my $from_group= $config->group($group_name)
-    or croak "There is no group named '$group_name' that ",
-      "can be used to resolve '$option_name'";
-
-  my $from= $from_group->value($option_name);
+  my $from;
+  
+  if ($group_name =~ "env")
+  {
+    $from = $ENV{$option_name};
+  } 
+  else
+  {
+    my $from_group= $config->group($group_name)
+      or croak "There is no group named '$group_name' that ",
+        "can be used to resolve '$option_name'";
+  
+    $from= $from_group->value($option_name);
+   }
+   
   $config->insert($group->name(), $option->name(), $from)
 }
 
@@ -521,6 +614,16 @@ sub post_fix_mysql_cluster_section {
     $config->insert('mysql_cluster'.$group->suffix(),
 		    'ndb_connectstring', $ndb_connectstring);
 
+    # Add ndb_connectstring to each ndbd connected to this
+    # cluster.
+    foreach my $ndbd ($config->like('cluster_config.ndbd.')) {
+      if ($ndbd->suffix() eq $group->suffix()) {
+        my $after = $ndbd->after('cluster_config.ndbd');
+        $config->insert("ndbd$after",
+                        'ndb_connectstring', $ndb_connectstring);
+      }
+    }
+
     # Add ndb_connectstring to each mysqld connected to this
     # cluster
     foreach my $mysqld ( $config->like('cluster_config.mysqld.')) {
@@ -580,11 +683,23 @@ sub run_section_rules {
 sub run_generate_sections_from_cluster_config {
   my ($self, $config)= @_;
 
-  my @options= ('ndb_mgmd', 'ndbd',
-		'mysqld', 'ndbapi');
-
   foreach my $group ( $config->like('cluster_config\.\w*$') ) {
 
+    # @options will be a list of ndbd, ndb_mgmd, mysqld, ndbapi ordered in the
+    # way that ndb_mgmd will assign node ids to nodes without id.
+    my @options;
+
+    foreach my $option ($group->options()) {
+      my $option_name = $option->name();
+      if ($option_name =~ /^(ndbd|ndb_mgmd|mysqld|ndbapi)$/)
+      {
+        push @options, $option_name;
+      }
+    }
+
+    # Keep track of next node id to use if not explicitly set.
+    my $next_nodeid = 1;
+
     # Keep track of current index per process type
     my %idxes;
     map { $idxes{$_}= 1; } @options;
@@ -611,6 +726,27 @@ sub run_generate_sections_from_cluster_config {
 	$config->insert("cluster_config.$option_name.$idx$suffix",
 			"HostName", $host);
 
+        # Predict how ndbd_mgm will assign node id for current node.
+        my $node = $config->group("cluster_config.$option_name.$idx$suffix");
+        my $nodeid = $node->if_exist('NodeId');
+        if (!$nodeid) {
+          $nodeid = $next_nodeid;
+          while ($config->{"cluster_config$suffix"}->
+                          {"ALLOCATED NODEIDS"}->
+                          {$nodeid}) {
+            $nodeid++;
+          }
+        }
+        $next_nodeid = $nodeid + 1;
+
+        # For data node set matching command line option --ndb-nodeid.
+        # If prediction of node id is wrong, and node id for another node type
+        # is used, that will cause testcase to fail during setup.
+        if ($option_name eq 'ndbd') {
+          $config->insert("$option_name.$idx$suffix",
+                          'ndb-nodeid', $nodeid);
+        }
+
 	if ($option_name eq 'mysqld'){
 	  my $datadir=
 	    $self->fix_cluster_dir($config,
@@ -670,6 +806,10 @@ sub new_config {
 			   @cluster_config_rules);
   $self->run_generate_sections_from_cluster_config($config);
 
+  $self->run_section_rules($config,
+                           'cluster_config\.\w',
+                           ({ 'CODE' => \&track_allocated_nodeid }));
+
   $self->run_section_rules($config,
 			   'cluster_config.ndb_mgmd.',
 			   @ndb_mgmd_rules);
@@ -681,6 +821,10 @@ sub new_config {
 			   'mysqld.',
 			   @mysqld_rules);
 
+  $self->run_section_rules($config,
+			   'memcached.',
+			   @memcached_rules);
+               
   # [mysqlbinlog] need additional settings
   $self->run_rules_for_group($config,
 			     $config->insert('mysqlbinlog'),
