@@ -1,5 +1,4 @@
-/* Copyright (c) 2000, 2003, 2004 MySQL AB
-   Use is subject to license terms
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,7 +36,7 @@ char wild_many='*';
 char wild_one='?';
 char wild_prefix=0; /* QQ this can potentially cause a SIGSEGV */
 
-int wild_compare(register const char *str, register const char *wildstr,
+int wild_compare(const char *str, const char *wildstr,
                  pbool str_is_pattern)
 {
   char cmp;
