@@ -1,25 +1,32 @@
-/* Copyright (C) 2005-2006 MySQL AB
+/* Copyright (c) 2005, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#ifdef __GNUC__
-#pragma implementation
-#endif
+#include "item_xmlfunc.h"
 
-#include "mysql_priv.h"
-#include "my_xml.h"
-#include "sp_pcontext.h"
+#include "my_xml.h"             // my_xml_node_type
+#include "item_cmpfunc.h"       // Item_bool_func
+#include "sp_pcontext.h"        // sp_variable
+#include "sql_class.h"          // THD
+#include "sql_parse.h"          // check_stack_overrun
 
 /*
   TODO: future development directions:
@@ -102,7 +109,7 @@ typedef struct my_xpath_st
   Item *rootelement;     /* The root element                          */
   String *context_cache; /* last context provider                     */
   String *pxml;          /* Parsed XML, an array of MY_XML_NODE       */
-  CHARSET_INFO *cs;      /* character set/collation string comparison */
+  const CHARSET_INFO *cs;/* character set/collation string comparison */
   int error;
 } MY_XPATH;
 
@@ -115,7 +122,7 @@ public:
   inline bool append_element(MY_XPATH_FLT *flt)
   {
     String *str= this;
-    return str->append((const char*)flt, (uint32) sizeof(MY_XPATH_FLT));
+    return str->append((const char*)flt, sizeof(MY_XPATH_FLT));
   }
   inline bool append_element(uint32 num, uint32 pos)
   {
@@ -125,7 +132,7 @@ public:
     add.size= 0;
     return append_element(&add);
   }
-  inline bool append_element(uint32 num, uint32 pos, uint32 size)
+  inline bool append_element(uint32 num, uint32 pos, size_t size)
   {
     MY_XPATH_FLT add;
     add.num= num;
@@ -137,7 +144,7 @@ public:
   { 
     return (MY_XPATH_FLT*) (ptr() + i * sizeof(MY_XPATH_FLT));
   }
-  inline uint32 numelements()
+  inline size_t numelements()
   {
     return length() / sizeof(MY_XPATH_FLT);
   }
@@ -187,7 +194,7 @@ public:
     fltend= (MY_XPATH_FLT*) (res->ptr() + res->length());
     String active;
     active.alloc(numnodes);
-    bzero((char*) active.ptr(), numnodes);
+    memset(const_cast<char*>(active.ptr()), 0, numnodes);
     for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
     {
       MY_XML_NODE *node;
@@ -218,6 +225,9 @@ public:
   {
     max_length= MAX_BLOB_WIDTH;
     collation.collation= pxml->charset();
+    // To avoid premature evaluation, mark all nodeset functions as non-const.
+    used_tables_cache= RAND_TABLE_BIT;
+    const_item_cache= false;
   }
   const char *func_name() const { return "nodeset"; }
 };
@@ -387,11 +397,10 @@ public:
 */
 class Item_xpath_cast_bool :public Item_int_func
 {
-  String *pxml;
   String tmp_value;
 public:
-  Item_xpath_cast_bool(Item *a, String *pxml_arg)
-    :Item_int_func(a), pxml(pxml_arg) {}
+  Item_xpath_cast_bool(Item *a)
+    :Item_int_func(a) {}
   const char *func_name() const { return "xpath_cast_bool"; }
   bool is_bool_func() { return 1; }
   longlong val_int()
@@ -435,11 +444,10 @@ public:
 
 class Item_func_xpath_position :public Item_int_func
 {
-  String *pxml;
   String tmp_value;
 public:
-  Item_func_xpath_position(Item *a, String *p)
-    :Item_int_func(a), pxml(p) {}
+  Item_func_xpath_position(Item *a)
+    :Item_int_func(a) {}
   const char *func_name() const { return "xpath_position"; }
   void fix_length_and_dec() { max_length=10; }
   longlong val_int()
@@ -454,11 +462,10 @@ public:
 
 class Item_func_xpath_count :public Item_int_func
 {
-  String *pxml;
   String tmp_value;
 public:
-  Item_func_xpath_count(Item *a, String *p)
-    :Item_int_func(a), pxml(p) {}
+  Item_func_xpath_count(Item *a)
+    :Item_int_func(a) {}
   const char *func_name() const { return "xpath_count"; }
   void fix_length_and_dec() { max_length=10; }
   longlong val_int()
@@ -488,7 +495,7 @@ public:
     String *res= args[0]->val_nodeset(&tmp_value);
     MY_XPATH_FLT *fltbeg= (MY_XPATH_FLT*) res->ptr();
     MY_XPATH_FLT *fltend= (MY_XPATH_FLT*) (res->ptr() + res->length());
-    uint numnodes= pxml->length() / sizeof(MY_XML_NODE);
+    size_t numnodes= pxml->length() / sizeof(MY_XML_NODE);
     MY_XML_NODE *nodebeg= (MY_XML_NODE*) pxml->ptr();
   
     for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
@@ -535,7 +542,7 @@ public:
     MY_XPATH_FLT *fltbeg= (MY_XPATH_FLT*) res->ptr();
     MY_XPATH_FLT *fltend= (MY_XPATH_FLT*) (res->ptr() + res->length());
     MY_XML_NODE *nodebeg= (MY_XML_NODE*) pxml->ptr();
-    uint numnodes= pxml->length() / sizeof(MY_XML_NODE);
+    size_t numnodes= pxml->length() / sizeof(MY_XML_NODE);
 
     for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
     {
@@ -570,13 +577,13 @@ String *Item_nodeset_func_rootelement::val_nodeset(String *nodeset)
 
 String * Item_nodeset_func_union::val_nodeset(String *nodeset)
 {
-  uint num_nodes= pxml->length() / sizeof(MY_XML_NODE);
+  size_t num_nodes= pxml->length() / sizeof(MY_XML_NODE);
   String set0, *s0= args[0]->val_nodeset(&set0);
   String set1, *s1= args[1]->val_nodeset(&set1);
   String both_str;
   both_str.alloc(num_nodes);
   char *both= (char*) both_str.ptr();
-  bzero((void*)both, num_nodes);
+  memset(both, 0, num_nodes);
   MY_XPATH_FLT *flt;
 
   fltbeg= (MY_XPATH_FLT*) s0->ptr();
@@ -663,7 +670,7 @@ String *Item_nodeset_func_ancestorbyname::val_nodeset(String *nodeset)
   prepare(nodeset);
   active_str.alloc(numnodes);
   active= (char*) active_str.ptr();
-  bzero((void*)active, numnodes);
+  memset(active, 0, numnodes);
   uint pos= 0;
 
   for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
@@ -705,7 +712,7 @@ String *Item_nodeset_func_parentbyname::val_nodeset(String *nodeset)
   prepare(nodeset);
   active_str.alloc(numnodes);
   active= (char*) active_str.ptr();
-  bzero((void*)active, numnodes);
+  memset(active, 0, numnodes);
   for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
   {
     uint j= nodebeg[flt->num].parent;
@@ -746,7 +753,8 @@ String *Item_nodeset_func_predicate::val_nodeset(String *str)
 {
   Item_nodeset_func *nodeset_func= (Item_nodeset_func*) args[0];
   Item_func *comp_func= (Item_func*)args[1];
-  uint pos= 0, size;
+  uint pos= 0;
+  size_t size;
   prepare(str);
   size= fltend - fltbeg;
   for (MY_XPATH_FLT *flt= fltbeg; flt < fltend; flt++)
@@ -767,7 +775,7 @@ String *Item_nodeset_func_elementbyindex::val_nodeset(String *nodeset)
   Item_nodeset_func *nodeset_func= (Item_nodeset_func*) args[0];
   prepare(nodeset);
   MY_XPATH_FLT *flt;
-  uint pos, size= fltend - fltbeg;
+  size_t pos, size= fltend - fltbeg;
   for (pos= 0, flt= fltbeg; flt < fltend; flt++)
   {
     nodeset_func->context_cache.length(0);
@@ -789,7 +797,7 @@ String *Item_nodeset_func_elementbyindex::val_nodeset(String *nodeset)
 static Item* nodeset2bool(MY_XPATH *xpath, Item *item)
 {
   if (item->type() == Item::XPATH_NODESET)
-    return new Item_xpath_cast_bool(item, xpath->pxml);
+    return new Item_xpath_cast_bool(item);
   return item;
 }
 
@@ -923,12 +931,12 @@ static Item *create_comparator(MY_XPATH *xpath,
   else if (a->type() == Item::XPATH_NODESET &&
            b->type() == Item::XPATH_NODESET)
   {
-    uint len= xpath->query.end - context->beg;
+    size_t len= xpath->query.end - context->beg;
     set_if_smaller(len, 32);
     my_printf_error(ER_UNKNOWN_ERROR,
                     "XPATH error: "
                     "comparison of two nodesets is not supported: '%.*s'",
-                    MYF(0), len, context->beg);
+                    MYF(0), static_cast<int>(len), context->beg);
 
     return 0; // TODO: Comparison of two nodesets
   }
@@ -941,14 +949,16 @@ static Item *create_comparator(MY_XPATH *xpath,
      in a loop through all of the nodes in the node set.
     */
 
-    Item *fake= new Item_string("", 0, xpath->cs);
+    Item_string *fake= new Item_string("", 0, xpath->cs);
+    /* Don't cache fake because its value will be changed during comparison.*/
+    fake->set_used_tables(RAND_TABLE_BIT);
     Item_nodeset_func *nodeset;
     Item *scalar, *comp;
     if (a->type() == Item::XPATH_NODESET)
     {
       nodeset= (Item_nodeset_func*) a;
       scalar= b;
-      comp= eq_func(oper, fake, scalar);
+      comp= eq_func(oper, (Item*)fake, scalar);
     }
     else
     {
@@ -971,12 +981,12 @@ static Item *create_comparator(MY_XPATH *xpath,
     The newly created item.
 */
 static Item* nametestfunc(MY_XPATH *xpath,
-                          int type, Item *arg, const char *beg, uint len)
+                          int type, Item *arg, const char *beg, size_t len)
 {
-  DBUG_ASSERT(arg != 0);
-  DBUG_ASSERT(arg->type() == Item::XPATH_NODESET);
-  DBUG_ASSERT(beg != 0);
-  DBUG_ASSERT(len > 0);
+  assert(arg != 0);
+  assert(arg->type() == Item::XPATH_NODESET);
+  assert(beg != 0);
+  assert(len > 0);
 
   Item *res;
   switch (type)
@@ -1096,7 +1106,7 @@ my_xpath_keyword(MY_XPATH *x,
   size_t length= end-beg;
   for (k= keyword_names; k->name; k++)
   {
-    if (length == k->length && !strncasecmp(beg, k->name, length))
+    if (length == k->length && !native_strncasecmp(beg, k->name, length))
     {
       x->extra= k->extra;
       return k->tok;
@@ -1142,13 +1152,23 @@ static Item *create_func_floor(MY_XPATH *xpath, Item **args, uint nargs)
 
 static Item *create_func_bool(MY_XPATH *xpath, Item **args, uint nargs)
 {
-  return new Item_xpath_cast_bool(args[0], xpath->pxml);
+  return new Item_xpath_cast_bool(args[0]);
 }
 
 
 static Item *create_func_number(MY_XPATH *xpath, Item **args, uint nargs)
 {
-  return new Item_xpath_cast_number(args[0]);
+  Item *arg;
+
+  if (nargs > 0)
+  {
+    arg= args[0];
+  }
+  else
+  {
+    arg= xpath->context != NULL ? xpath->context : xpath->rootelement;
+  }
+  return new Item_xpath_cast_number(arg);
 }
 
 
@@ -1161,28 +1181,27 @@ static Item *create_func_string_length(MY_XPATH *xpath, Item **args, uint nargs)
 
 static Item *create_func_round(MY_XPATH *xpath, Item **args, uint nargs)
 {
-  return new Item_func_round(args[0], new Item_int((char*)"0",0,1),0);
+  return new Item_func_round(args[0], new Item_int_0(), 0);
 }
 
 
 static Item *create_func_last(MY_XPATH *xpath, Item **args, uint nargs)
 {
   return xpath->context ? 
-         new Item_func_xpath_count(xpath->context, xpath->pxml) : NULL;
+         new Item_func_xpath_count(xpath->context) : NULL;
 }
 
 
 static Item *create_func_position(MY_XPATH *xpath, Item **args, uint nargs)
 {
   return xpath->context ? 
-         new Item_func_xpath_position(xpath->context, xpath->pxml) : NULL;
+         new Item_func_xpath_position(xpath->context) : NULL;
 }
 
 
 static Item *create_func_contains(MY_XPATH *xpath, Item **args, uint nargs)
 {
-  return new Item_xpath_cast_bool(new Item_func_locate(args[0], args[1]),
-                                  xpath->pxml);
+  return new Item_xpath_cast_bool(new Item_func_locate(args[0], args[1]));
 }
 
 
@@ -1205,7 +1224,7 @@ static Item *create_func_count(MY_XPATH *xpath, Item **args, uint nargs)
 {  
   if (args[0]->type() != Item::XPATH_NODESET)
     return 0;
-  return new Item_func_xpath_count(args[0], xpath->pxml);
+  return new Item_func_xpath_count(args[0]);
 }
 
 
@@ -1296,7 +1315,7 @@ MY_XPATH_FUNC *
 my_xpath_function(const char *beg, const char *end)
 {
   MY_XPATH_FUNC *k, *function_names;
-  uint length= end-beg;
+  size_t length= end-beg;
   switch (length)
   {
     case 1: return 0;
@@ -1307,7 +1326,7 @@ my_xpath_function(const char *beg, const char *end)
     default: function_names= my_func_names;
   }
   for (k= function_names; k->name; k++)
-    if (k->create && length == k->length && !strncasecmp(beg, k->name, length))
+    if (k->create && length == k->length && !native_strncasecmp(beg, k->name, length))
       return k;
   return NULL;
 }
@@ -1327,7 +1346,7 @@ my_xpath_lex_init(MY_XPATH_LEX *lex,
 static void
 my_xpath_init(MY_XPATH *xpath)
 {
-  bzero((void*)xpath, sizeof(xpath[0]));
+  memset(xpath, 0, sizeof(xpath[0]));
 }
 
 
@@ -1354,7 +1373,7 @@ my_xpath_lex_scan(MY_XPATH *xpath,
                   MY_XPATH_LEX *lex, const char *beg, const char *end)
 {
   int ch, ctype, length;
-  for ( ; beg < end && *beg == ' ' ; beg++); // skip leading spaces
+  for ( ; beg < end && *beg == ' ' ; beg++) ; // skip leading spaces
   lex->beg= beg;
   
   if (beg >= end)
@@ -1423,7 +1442,7 @@ my_xpath_lex_scan(MY_XPATH *xpath,
 
   if (my_xdigit(ch)) // a sequence of digits
   {
-    for ( ; beg < end && my_xdigit(*beg) ; beg++);
+    for ( ; beg < end && my_xdigit(*beg) ; beg++) ;
     lex->end= beg;
     lex->term= MY_XPATH_LEX_DIGITS;
     return;
@@ -1431,7 +1450,7 @@ my_xpath_lex_scan(MY_XPATH *xpath,
 
   if (ch == '"' || ch == '\'')  // a string: either '...' or "..."
   {
-    for ( ; beg < end && *beg != ch ; beg++);
+    for ( ; beg < end && *beg != ch ; beg++) ;
     if (beg < end)
     {
       lex->end= beg+1;
@@ -1539,6 +1558,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath);
 
     [1] LocationPath ::=   RelativeLocationPath
                          | AbsoluteLocationPath
+    [3] RelativeLocationPath ::= RelativeLocationPath '/' Step
 
   RETURN
     1 - success
@@ -1648,6 +1668,12 @@ static int my_xpath_parse_RelativeLocationPath(MY_XPATH *xpath)
   [4] Step ::=   AxisSpecifier NodeTest Predicate*
                | AbbreviatedStep
   [8] Predicate ::= '[' PredicateExpr ']'
+  [9] PredicateExpr ::= Expr (RECURSIVE)
+  [14] Expr ::= OrExpr
+
+  reduced to:
+
+  [8b] Predicate ::= '[' OrExpr ']' (RECURSIVE)
 
   RETURN
     1 - success
@@ -1828,10 +1854,16 @@ static int my_xpath_parse_AbbreviatedStep(MY_XPATH *xpath)
   SYNOPSYS
 
   [15] PrimaryExpr ::= VariableReference	
-                       | '(' Expr ')'	
+                       | '(' Expr ')'   (RECURSIVE)
                        | Literal	
                        | Number	
                        | FunctionCall
+  [14] Expr ::= OrExpr
+
+  reduced to:
+
+  [15b] PrimaryExpr ::= '(' OrExpr ')'  (RECURSIVE)
+
   RETURN
     1 - success
     0 - failure
@@ -1867,7 +1899,12 @@ static int my_xpath_parse_PrimaryExpr(MY_XPATH *xpath)
 
   SYNOPSYS
     [16] FunctionCall ::= FunctionName '(' ( Argument ( ',' Argument )* )? ')'
-    [17] Argument      ::= Expr
+    [17] Argument      ::= Expr (RECURSIVE)
+    [14] Expr ::= OrExpr
+ 
+    reduced to:
+ 
+    [16b] FunctionCall ::= FunctionName '(' ( OrExpr ( ',' OrExpr )* )? ')' (RECURSIVE)
 
   RETURN
     1 - success
@@ -1963,20 +2000,43 @@ static int my_xpath_parse_UnionExpr(MY_XPATH *xpath)
 static int
 my_xpath_parse_FilterExpr_opt_slashes_RelativeLocationPath(MY_XPATH *xpath)
 {
+  Item *context= xpath->context;
+  int rc;
+
   if (!my_xpath_parse_FilterExpr(xpath))
     return 0;
 
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_SLASH))
     return 1;
 
-  my_xpath_parse_term(xpath, MY_XPATH_LEX_SLASH);
-  return my_xpath_parse_RelativeLocationPath(xpath);
+  if (xpath->item->type() != Item::XPATH_NODESET)
+  {
+    xpath->lasttok= xpath->prevtok;
+    xpath->error= 1;
+    return 0;
+  }
+
+  /*
+    The context for the next relative path is the nodeset
+    returned by FilterExpr
+  */
+  xpath->context= xpath->item;
+
+  /* treat double slash (//) as /descendant-or-self::node()/ */
+  if (my_xpath_parse_term(xpath, MY_XPATH_LEX_SLASH))
+    xpath->context= new Item_nodeset_func_descendantbyname(xpath->context,
+                                                           "*", 1, xpath->pxml, 1);
+  rc= my_xpath_parse_RelativeLocationPath(xpath);
+
+  /* push back the context and restore the item */
+  xpath->item= xpath->context;
+  xpath->context= context;
+  return rc;
 }
 static int my_xpath_parse_PathExpr(MY_XPATH *xpath)
 {
   return my_xpath_parse_LocationPath(xpath) || 
          my_xpath_parse_FilterExpr_opt_slashes_RelativeLocationPath(xpath);
-         
 }
 
 
@@ -2016,6 +2076,12 @@ static int my_xpath_parse_FilterExpr(MY_XPATH *xpath)
 */
 static int my_xpath_parse_OrExpr(MY_XPATH *xpath)
 {
+  THD *thd= current_thd;
+  uchar stack_top;
+
+  if (check_stack_overrun(thd, STACK_MIN_SIZE, &stack_top))
+   return 1;
+
   if (!my_xpath_parse_AndExpr(xpath))
     return 0;
 
@@ -2305,6 +2371,12 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath)
 */
 static int my_xpath_parse_UnaryExpr(MY_XPATH *xpath)
 {
+  THD *thd= current_thd;
+  uchar stack_top;
+
+  if (check_stack_overrun(thd, STACK_MIN_SIZE, &stack_top))
+    return 0;
+
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_MINUS))
     return my_xpath_parse_UnionExpr(xpath);
   if (!my_xpath_parse_UnaryExpr(xpath))
@@ -2456,18 +2528,19 @@ my_xpath_parse_VariableReference(MY_XPATH *xpath)
   name.str= (char*) xpath->prevtok.beg;
   
   if (user_var)
-    xpath->item= new Item_func_get_user_var(name);
+    xpath->item= new Item_func_get_user_var(Name_string(name, false));
   else
   {
-    sp_variable_t *spv;
+    sp_variable *spv;
     sp_pcontext *spc;
     LEX *lex;
     if ((lex= current_thd->lex) &&
-        (spc= lex->spcont) &&
-        (spv= spc->find_variable(&name)))
+        (spc= lex->get_sp_current_parsing_ctx()) &&
+        (spv= spc->find_variable(name, false)))
     {
-      Item_splocal *splocal= new Item_splocal(name, spv->offset, spv->type, 0);
-#ifndef DBUG_OFF
+      Item_splocal *splocal= new Item_splocal(Name_string(name, false),
+                                              spv->offset, spv->type, 0);
+#ifndef NDEBUG
       if (splocal)
         splocal->m_sp= lex->sphead;
 #endif
@@ -2476,11 +2549,11 @@ my_xpath_parse_VariableReference(MY_XPATH *xpath)
     else
     {
       xpath->item= NULL;
-      DBUG_ASSERT(xpath->query.end > dollar_pos);
-      uint len= xpath->query.end - dollar_pos;
+      assert(xpath->query.end > dollar_pos);
+      size_t len= xpath->query.end - dollar_pos;
       set_if_smaller(len, 32);
-      my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'", 
-                      MYF(0), len, dollar_pos);
+      my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'",
+                      MYF(0), static_cast<int>(len), dollar_pos);
     }
   }
   return xpath->item ? 1 : 0;
@@ -2504,8 +2577,8 @@ my_xpath_parse_NodeTest_QName(MY_XPATH *xpath)
 {
   if (!my_xpath_parse_QName(xpath))
     return 0;
-  DBUG_ASSERT(xpath->context);
-  uint len= xpath->prevtok.end - xpath->prevtok.beg;
+  assert(xpath->context);
+  size_t len= xpath->prevtok.end - xpath->prevtok.beg;
   xpath->context= nametestfunc(xpath, xpath->axis, xpath->context,
                                xpath->prevtok.beg, len);
   return 1;
@@ -2515,7 +2588,7 @@ my_xpath_parse_NodeTest_asterisk(MY_XPATH *xpath)
 {
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_ASTERISK))
     return 0;
-  DBUG_ASSERT(xpath->context);
+  assert(xpath->context);
   xpath->context= nametestfunc(xpath, xpath->axis, xpath->context, "*", 1);
   return 1;
 }
@@ -2555,13 +2628,9 @@ my_xpath_parse(MY_XPATH *xpath, const char *str, const char *strend)
 
 void Item_xml_str_func::fix_length_and_dec()
 {
-  String *xp, tmp;
-  MY_XPATH xpath;
-  int rc;
-
   nodeset_func= 0;
 
-  if (agg_arg_charsets(collation, args, arg_count, MY_COLL_CMP_CONV, 1))
+  if (agg_arg_charsets_for_comparison(collation, args, arg_count))
     return;
 
   if (collation.collation->mbminlen > 1)
@@ -2573,36 +2642,47 @@ void Item_xml_str_func::fix_length_and_dec()
     return;
   }
 
-  if (!args[1]->const_item())
+  if (!args[1]->const_during_execution())
   {
     my_printf_error(ER_UNKNOWN_ERROR,
                     "Only constant XPATH queries are supported", MYF(0));
     return;
   }
 
-  if (!(xp= args[1]->val_str(&tmp)))
+  if (args[1]->const_item())
+    parse_xpath(args[1]);
+
+  max_length= MAX_BLOB_WIDTH;
+}
+
+void Item_xml_str_func::parse_xpath(Item* xpath_expr)
+{
+  String *xp;
+  MY_XPATH xpath;
+
+  if (!(xp= xpath_expr->val_str(&xpath_tmp_value)))
     return;
+
   my_xpath_init(&xpath);
   xpath.cs= collation.collation;
   xpath.debug= 0;
   xpath.pxml= &pxml;
   pxml.set_charset(collation.collation);
 
-  rc= my_xpath_parse(&xpath, xp->ptr(), xp->ptr() + xp->length());
+  int rc= my_xpath_parse(&xpath, xp->ptr(), xp->ptr() + xp->length());
 
   if (!rc)
   {
-    uint clen= xpath.query.end - xpath.lasttok.beg;
+    size_t clen= xpath.query.end - xpath.lasttok.beg;
     set_if_smaller(clen, 32);
     my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.*s'",
-                    MYF(0), clen, xpath.lasttok.beg);
+                    MYF(0), static_cast<int>(clen), xpath.lasttok.beg);
     return;
   }
 
   nodeset_func= xpath.item;
   if (nodeset_func)
     nodeset_func->fix_fields(current_thd, &nodeset_func);
-  max_length= MAX_BLOB_WIDTH;
 }
 
 
@@ -2654,13 +2734,17 @@ extern "C" int xml_enter(MY_XML_PARSER *st,const char *attr, size_t len);
 int xml_enter(MY_XML_PARSER *st,const char *attr, size_t len)
 {
   MY_XML_USER_DATA *data= (MY_XML_USER_DATA*)st->user_data;
-  uint numnodes= data->pxml->length() / sizeof(MY_XML_NODE);
+  size_t numnodes= data->pxml->length() / sizeof(MY_XML_NODE);
   MY_XML_NODE node;
 
   node.parent= data->parent; // Set parent for the new node to old parent
   data->parent= numnodes;    // Remember current node as new parent
+  assert(data->level < MAX_LEVEL);
   data->pos[data->level]= numnodes;
-  node.level= data->level++;
+  if (data->level < MAX_LEVEL - 1)
+    node.level= data->level++;
+  else
+    return MY_XML_ERROR;
   node.type= st->current_node_type; // TAG or ATTR
   node.beg= attr;
   node.end= attr + len;
@@ -2713,7 +2797,7 @@ extern "C" int xml_leave(MY_XML_PARSER *st,const char *attr, size_t len);
 int xml_leave(MY_XML_PARSER *st,const char *attr, size_t len)
 {
   MY_XML_USER_DATA *data= (MY_XML_USER_DATA*)st->user_data;
-  DBUG_ASSERT(data->level > 0);
+  assert(data->level > 0);
   data->level--;
 
   MY_XML_NODE *nodes= (MY_XML_NODE*) data->pxml->ptr();
@@ -2766,7 +2850,7 @@ String *Item_xml_str_func::parse_xml(String *raw_xml, String *parsed_xml_buf)
                 my_xml_error_lineno(&p) + 1,
                 (ulong) my_xml_error_pos(&p) + 1,
                 my_xml_error_string(&p));
-    push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+    push_warning_printf(current_thd, Sql_condition::SL_WARNING,
                         ER_WRONG_VALUE,
                         ER(ER_WRONG_VALUE), "XML", buf);
   }
@@ -2779,14 +2863,18 @@ String *Item_xml_str_func::parse_xml(String *raw_xml, String *parsed_xml_buf)
 String *Item_func_xml_extractvalue::val_str(String *str)
 {
   String *res;
+  null_value= 0;
+  if (!nodeset_func)
+    parse_xpath(args[1]);
+  tmp_value.set("", 0, pxml.charset());
   if (!nodeset_func ||
       !(res= args[0]->val_str(str)) || 
-      !parse_xml(res, &pxml))
+      !parse_xml(res, &pxml) ||
+      !(res= nodeset_func->val_str(&tmp_value)))
   {
     null_value= 1;
     return 0;
   }
-  res= nodeset_func->val_str(&tmp_value);
   return res;  
 }
 
@@ -2795,6 +2883,9 @@ String *Item_func_xml_update::val_str(String *str)
 {
   String *res, *nodeset, *rep;
 
+  null_value= 0;
+  if (!nodeset_func)
+    parse_xpath(args[1]);
   if (!nodeset_func || 
       !(res= args[0]->val_str(str)) ||
       !(rep= args[2]->val_str(&tmp_value3)) ||
