@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -85,6 +85,8 @@ public:
 
     _CMVMIMin = 103000,
     SetSchedulerResponsiveness = 103000,
+    EnableEventLoggerDebug = 103001,
+    DisableEventLoggerDebug = 103002,
     _CMVMIMax = 103099,
 
     // 1 QMGR Dump information about phase 1 variables
@@ -109,6 +111,8 @@ public:
     NdbfsDumpAllFiles = 401,
     NdbfsDumpOpenFiles = 402,
     NdbfsDumpIdleFiles = 403,
+    NdbfsDumpRequests = 406,
+    NdbfsErrorInsert = 407,
     CmvmiSchedulerExecutionTimer = 502,
     CmvmiRealtimeScheduler = 503,
     CmvmiExecuteLockCPU = 504,
@@ -135,6 +139,7 @@ public:
     AccDumpFreeOpRecs = 2405,
     AccDumpNotFreeOpRecs = 2406,
     DumpPageMemory = 1000, // Acc & TUP
+    DumpPageMemoryOnFail = 1001,
     TcDumpSetOfScanFragRec = 2500,
     TcDumpOneScanFragRec = 2501,
     TcDumpSetOfScanRec = 2502,
