@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -47,10 +47,12 @@ NdbPack::Error::set_error(int code, int line) const
   m_error_code = code;
   m_error_line = line;
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
   const char* p = NdbEnv_GetEnv("NDB_PACK_ABORT_ON_ERROR", (char*)0, 0);
   if (p != 0 && strchr("1Y", p[0]) != 0)
     require(false);
 #endif
+#endif
 }
 
 void
@@ -132,53 +134,63 @@ static const int g_ndb_pack_type_info_cnt =
 int
 NdbPack::Type::complete()
 {
-  if (m_typeId == 0)
+  if (unlikely(m_typeId == 0))
   {
     set_error(TypeNotSet, __LINE__);
     return -1;
   }
-  if (m_typeId >= g_ndb_pack_type_info_cnt)
+  if (unlikely(m_typeId >= g_ndb_pack_type_info_cnt))
   {
     set_error(TypeNotSet, __LINE__);
     return -1;
   }
   const Ndb_pack_type_info& info = g_ndb_pack_type_info[m_typeId];
-  if (!info.m_supported)
+  if (unlikely(!info.m_supported))
   {
     set_error(TypeNotSupported, __LINE__);
     return -1;
   }
-  if (m_byteSize == 0)
+  if (unlikely(m_byteSize == 0))
   {
     set_error(TypeSizeZero, __LINE__);
     return -1;
   }
-  if (info.m_fixSize != 0 && m_byteSize != info.m_fixSize)
+  if (unlikely(info.m_fixSize != 0 && m_byteSize != info.m_fixSize))
   {
     set_error(TypeFixSizeInvalid, __LINE__);
     return -1;
   }
-  if (!(m_nullable <= 1))
+  if (unlikely(!(m_nullable <= 1)))
   {
     set_error(TypeNullableNotBool, __LINE__);
     return -1;
   }
-  if (info.m_charType && m_csNumber == 0)
+  if (unlikely(info.m_charType))
   {
-    set_error(CharsetNotSpecified, __LINE__);
-    return -1;
-  }
-  if (info.m_charType && all_charsets[m_csNumber] == 0)
-  {
-    CHARSET_INFO* cs = get_charset(m_csNumber, MYF(0));
-    if (cs == 0)
+    if (unlikely(m_csNumber == 0))
+    {
+      set_error(CharsetNotSpecified, __LINE__);
+      return -1;
+    }
+
+    if (unlikely(m_csNumber >= NDB_ARRAY_SIZE(all_charsets)))
     {
       set_error(CharsetNotFound, __LINE__);
       return -1;
     }
-    all_charsets[m_csNumber] = cs; // yes caller must do this
+
+    if (unlikely(all_charsets[m_csNumber] == 0))
+    {
+      CHARSET_INFO* cs = get_charset(m_csNumber, MYF(0));
+      if (unlikely(cs == 0))
+      {
+        set_error(CharsetNotFound, __LINE__);
+        return -1;
+      }
+      all_charsets[m_csNumber] = cs; // yes caller must do this
+    }
   }
-  if (!info.m_charType && m_csNumber != 0)
+  if (unlikely(!info.m_charType && m_csNumber != 0))
   {
     set_error(CharsetNotAllowed, __LINE__);
     return -1;
@@ -196,7 +208,7 @@ NdbPack::Spec::add(Type type)
   Uint32 nullable_cnt = m_nullableCnt;
   Uint32 varsize_cnt = m_varsizeCnt;
   Uint32 max_byte_size = m_maxByteSize;
-  if (type.complete() == -1)
+  if (unlikely(type.complete() == -1))
   {
     set_error(type);
     return -1;
@@ -212,7 +224,7 @@ NdbPack::Spec::add(Type type)
     varsize_cnt++;
   }
   max_byte_size += type.m_byteSize;
-  if (cnt >= m_bufMaxCnt)
+  if (unlikely(cnt >= m_bufMaxCnt))
   {
     set_error(SpecBufOverflow, __LINE__);
     return -1;
@@ -231,7 +243,7 @@ NdbPack::Spec::add(Type type, Uint32 cnt)
 {
   for (Uint32 i = 0; i < cnt; i++)
   {
-    if (add(type) == -1)
+    if (unlikely(add(type) == -1))
       return -1;
   }
   return 0;
@@ -278,7 +290,7 @@ NdbPack::Iter::desc(const Uint8* item)
     return -1;
   }
   const Uint32 itemLen = lenBytes + bareLen;
-  if (itemLen > type.m_byteSize)
+  if (unlikely(itemLen > type.m_byteSize))
   {
     set_error(DataValueOverflow, __LINE__);
     return -1;
@@ -305,6 +317,94 @@ NdbPack::Iter::desc_null()
   return 0;
 }
 
+/**
+ * Initialise a DataArray from a NdbPack::BoundC object.
+ */
+void NdbPack::DataArray::init_bound(const BoundC& b,
+                                    const Uint32 cnt)
+{
+  m_cnt = cnt;
+  const DataC& data = b.m_data;
+  Iter iter(data);
+  for (Uint32 i = 0; i < cnt; i++)
+  {
+    data.desc(iter);
+    m_entries[i].m_data_ptr = (Uint8*)&data.m_buf[iter.m_itemPos];
+    m_entries[i].m_data_len = iter.m_itemLen;
+  }
+  m_null_cnt = iter.m_nullCnt;
+}
+
+/**
+ * Initialise a DataArray object from Attrinfo retrieved
+ * from DBTUP.
+ */
+void
+NdbPack::DataArray::init_poai(const Uint32 *buffer,
+                              const Uint32 cnt)
+{
+  Uint32 inx = 0;
+  m_cnt = cnt;
+  m_null_cnt = 0;
+  for (Uint32 i = 0; i < cnt; i++)
+  {
+    const AttributeHeader ah =
+      *(const AttributeHeader*)&buffer[inx++];
+
+    if (!ah.isNULL())
+    {
+      Uint32 byte_size = ah.getByteSize();
+      Uint32 word_size = ah.getDataSize();
+      m_entries[i].m_data_len = byte_size;
+      m_entries[i].m_data_ptr = (Uint8*)&buffer[inx];
+      inx += word_size;
+    }
+    else
+    {
+      m_null_cnt++;
+      m_entries[i].m_data_ptr = 0;
+      m_entries[i].m_data_len = 0;
+    }
+  }
+}
+
+int
+NdbPack::DataArray::cmp(const Spec* spec,
+                        const DataArray* d2,
+                        const Uint32 cnt) const
+{
+  int res = 0;
+  for (Uint32 i = 0; i < cnt; i++)
+  {
+    const Type& type = spec->m_buf[i];
+    const Uint8* p1 = m_entries[i].m_data_ptr;
+    const Uint32 n1 = m_entries[i].m_data_len;
+    const NdbSqlUtil::Type& sqlType = getSqlType(type.m_typeId);
+    const Uint8* p2 = d2->m_entries[i].m_data_ptr;
+    const Uint32 n2 = d2->m_entries[i].m_data_len;
+    CHARSET_INFO* cs = all_charsets[type.m_csNumber];
+    if (n1 != 0)
+    {
+      if (n2 != 0)
+      {
+        res = (*sqlType.m_cmp)(cs, p1, n1, p2, n2);
+      }
+      else
+      {
+        res = +1;
+      }
+    }
+    else
+    {
+      if (n2 != 0)
+        res = -1;
+    }
+    if (res != 0)
+      break;
+  }
+  return res;
+}
+
 int
 NdbPack::Iter::cmp(const Iter& r2, const Uint8* buf1, const Uint8* buf2) const
 {
@@ -360,7 +460,7 @@ NdbPack::DataC::desc(Iter& r) const
     const Uint8& the_byte = m_buf[byte_pos];
     if ((the_byte & bit_mask) != 0)
     {
-      if (r.desc_null() == -1)
+      if (unlikely(r.desc_null() == -1))
       {
         set_error(r);
         return -1;
@@ -370,7 +470,7 @@ NdbPack::DataC::desc(Iter& r) const
   }
   const Uint32 pos = r.m_itemPos + r.m_itemLen;
   const Uint8* item = &m_buf[pos];
-  if (r.desc(item) == -1)
+  if (unlikely(r.desc(item) == -1))
   {
     set_error(r);
     return -1;
@@ -408,7 +508,7 @@ NdbPack::Data::add(const void* data, Uint32* len_out)
   assert(data != 0);
   const Uint8* item = (const Uint8*)data;
   const Uint32 i = m_cnt; // item index
-  if (i >= m_spec.m_cnt)
+  if (unlikely(i >= m_spec.m_cnt))
   {
     set_error(DataCntOverflow, __LINE__);
     return -1;
@@ -416,12 +516,12 @@ NdbPack::Data::add(const void* data, Uint32* len_out)
   Iter& r = m_iter;
   assert(r.m_cnt == i);
   const Uint32 fullLen = m_varBytes + r.m_itemPos + r.m_itemLen;
-  if (r.desc(item) == -1)
+  if (unlikely(r.desc(item) == -1))
   {
     set_error(r);
     return -1;
   }
-  if (fullLen + r.m_itemLen > m_bufMaxLen)
+  if (unlikely(fullLen + r.m_itemLen > m_bufMaxLen))
   {
     set_error(DataBufOverflow, __LINE__);
     return -1;
@@ -440,7 +540,7 @@ NdbPack::Data::add(const void* data, Uint32 cnt, Uint32* len_out)
   for (Uint32 i = 0; i < cnt; i++)
   {
     Uint32 len;
-    if (add(data_ptr, &len) == -1)
+    if (unlikely(add(data_ptr, &len) == -1))
       return -1;
     if (data != 0)
       data_ptr += len;
@@ -454,14 +554,14 @@ int
 NdbPack::Data::add_null(Uint32* len_out)
 {
   const Uint32 i = m_cnt; // item index
-  if (i >= m_spec.m_cnt)
+  if (unlikely(i >= m_spec.m_cnt))
   {
     set_error(DataCntOverflow, __LINE__);
     return -1;
   }
   Iter& r = m_iter;
   assert(r.m_cnt == i);
-  if (r.desc_null() == -1)
+  if (unlikely(r.desc_null() == -1))
   {
     set_error(r);
     return -1;
@@ -470,7 +570,7 @@ NdbPack::Data::add_null(Uint32* len_out)
   if (!m_allNullable)
   {
     const Type& type = m_spec.m_buf[i];
-    if (!type.m_nullable)
+    if (unlikely(!type.m_nullable))
     {
       set_error(DataNotNullable, __LINE__);
       return -1;
@@ -499,7 +599,7 @@ NdbPack::Data::add_null(Uint32 cnt, Uint32* len_out)
   for (Uint32 i = 0; i < cnt; i++)
   {
     Uint32 len;
-    if (add_null(&len) == -1)
+    if (unlikely(add_null(&len) == -1))
       return -1;
     len_tot += len;
   }
@@ -513,15 +613,15 @@ NdbPack::Data::add_poai(const Uint32* poai, Uint32* len_out)
   const AttributeHeader ah = *(const AttributeHeader*)&poai[0];
   if (!ah.isNULL())
   {
-    if (add(&poai[1], len_out) == -1)
+    if (unlikely(add(&poai[1], len_out) == -1))
       return -1;
   }
   else
   {
-    if (add_null(len_out) == -1)
+    if (unlikely(add_null(len_out) == -1))
       return -1;
   }
-  if (ah.getByteSize() != *len_out)
+  if (unlikely(ah.getByteSize() != *len_out))
   {
     set_error(InvalidAttrInfo, __LINE__);
     return -1;
@@ -536,7 +636,7 @@ NdbPack::Data::add_poai(const Uint32* poai, Uint32 cnt, Uint32* len_out)
   for (Uint32 i = 0; i < cnt; i++)
   {
     Uint32 len;
-    if (add_poai(poai, &len) == -1)
+    if (unlikely(add_poai(poai, &len) == -1))
       return -1;
     len_tot += len;
     poai += 1 + (len + 3) / 4;
@@ -583,10 +683,10 @@ NdbPack::Data::desc_all(Uint32 cnt, Endian::Value from_endian)
   for (Uint32 i = 0; i < cnt; i++)
   {
     m_cnt++;
-    if (desc(m_iter) == -1)
+    if (unlikely(desc(m_iter) == -1))
       return -1;
   }
-  if (finalize() == -1)
+  if (unlikely(finalize() == -1))
     return -1;
   return 0;
 }
@@ -599,23 +699,23 @@ NdbPack::Data::copy(const DataC& d2)
   const Uint32 cnt2 = d2.m_cnt;
   for (Uint32 i = 0; i < cnt2; i++)
   {
-    if (d2.desc(r2) == -1)
+    if (unlikely(d2.desc(r2) == -1))
       return -1;
     Uint32 len_out = ~(Uint32)0;
     if (r2.m_itemLen != 0)
     {
-      if (add(&d2.m_buf[r2.m_itemPos], &len_out) == -1)
+      if (unlikely(add(&d2.m_buf[r2.m_itemPos], &len_out) == -1))
           return -1;
       assert(len_out == r2.m_itemLen);
     }
     else
     {
-      if (add_null(&len_out) == -1)
+      if (unlikely(add_null(&len_out) == -1))
         return -1;
       assert(len_out ==0);
     }
   }
-  if (finalize() == -1)
+  if (unlikely(finalize() == -1))
     return -1;
   return 0;
 }
@@ -627,7 +727,7 @@ NdbPack::Data::convert_impl(Endian::Value to_endian)
   Iter r(*this);
   for (Uint32 i = 0; i < m_cnt; i++)
   {
-    if (DataC::desc(r) == -1)
+    if (unlikely(DataC::desc(r) == -1))
     {
       set_error(r);
       return -1;
@@ -650,12 +750,12 @@ NdbPack::Data::convert_impl(Endian::Value to_endian)
 int
 NdbPack::BoundC::finalize(int side)
 {
-  if (m_data.m_cnt == 0 && side != 0)
+  if (unlikely(m_data.m_cnt == 0 && side != 0))
   {
     set_error(BoundEmptySide, __LINE__);
     return -1;
   }
-  if (m_data.m_cnt != 0 && side != -1 && side != +1)
+  if (unlikely(m_data.m_cnt != 0 && side != -1 && side != +1))
   {
     set_error(BoundNonemptySide, __LINE__);
     return -1;
