@@ -1,24 +1,34 @@
-/* Copyright (c) 2008-2010 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* 
+   Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 #ifndef DBINFO_H
 #define DBINFO_H
 
 #include <SimulatedBlock.hpp>
 
+#define JAM_FILE_ID 454
+
+
 class Dbinfo : public SimulatedBlock
 {
 public:
@@ -39,6 +49,12 @@ protected:
 
   void execINCL_NODEREQ(Signal* signal);
   void execNODE_FAILREP(Signal* signal);
+
+ private:
+   Ndbinfo::Counts counts;
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
