@@ -39,14 +39,14 @@ void Dbtup::execSEND_PACKED(Signal* signal)
   Uint32 i;
   Uint32 TpackedListIndex= cpackedListIndex;
   bool present = false;
-  jamEntry();
   for (i= 0; i < TpackedListIndex; i++) {
     jam();
     hostId= cpackedList[i];
     ndbrequire((hostId - 1) < (MAX_NODES - 1)); // Also check not zero
-    Uint32 TpacketTA= hostBuffer[hostId].noOfPacketsTA;
+    HostBuffer* const buffer = &hostBuffer[hostId];
+    Uint32 TpacketTA= buffer->noOfPacketsTA;
     if (TpacketTA != 0) {
-      jam();
+      jamDebug();
 
       if (ERROR_INSERTED(4037))
       {
@@ -55,107 +55,199 @@ void Dbtup::execSEND_PACKED(Signal* signal)
         if (!present)
         {
           /* First valid packed data in this pass */
-          jam();
+          jamDebug();
           present = true;
           cerrorPackedDelay++;
           
           if ((cerrorPackedDelay % 10) != 0)
           {
             /* Skip it */
-            jam();
+            jamDebug();
             return;
           }
         }
       }
-      BlockReference TBref= numberToRef(API_PACKED, hostId);
-      Uint32 TpacketLen= hostBuffer[hostId].packetLenTA;
+      const BlockReference TBref= numberToRef(API_PACKED, hostId);
+      const Uint32 TpacketLen= buffer->packetLenTA;
       MEMCOPY_NO_WORDS(&signal->theData[0],
-                       &hostBuffer[hostId].packetBufferTA[0],
+                       &buffer->packetBufferTA[0],
                        TpacketLen);
       sendSignal(TBref, GSN_TRANSID_AI, signal, TpacketLen, JBB);
-      hostBuffer[hostId].noOfPacketsTA= 0;
-      hostBuffer[hostId].packetLenTA= 0;
+      buffer->noOfPacketsTA= 0;
+      buffer->packetLenTA= 0;
     }
-    hostBuffer[hostId].inPackedList= false;
+    buffer->inPackedList= false;
   }//for
   cpackedListIndex= 0;
 }
 
+/**
+ * Copy a TRANSID_AI signal, which alread has its header constructed in 'signal',
+ * into a packed buffer structure.
+ *
+ * Prereq:
+ *  - Signal should be sufficiently small to allow it to be 'packed'
+ *  - Buffer should have sufficient free space for the signal.
+ */
 void Dbtup::bufferTRANSID_AI(Signal* signal, BlockReference aRef,
-                             Uint32 Tlen)
+                             const Uint32 *dataBuf,
+                             Uint32 lenOfData)
 {
-  if (Tlen == 3)
-    return;
-  
-  Uint32 hostId= refToNode(aRef);
-  Uint32 Theader= ((refToBlock(aRef) << 16)+(Tlen-3));
-  
-  ndbrequire(hostId < MAX_NODES);
-  Uint32 TpacketLen= hostBuffer[hostId].packetLenTA;
-  Uint32 TnoOfPackets= hostBuffer[hostId].noOfPacketsTA;
-  Uint32 sig0= signal->theData[0];
-  Uint32 sig1= signal->theData[1];
-  Uint32 sig2= signal->theData[2];
+  ndbassert(lenOfData > 0);
+  ndbassert(TransIdAI::HeaderLength+lenOfData+1 <= 25);
 
-  BlockReference TBref= numberToRef(API_PACKED, hostId);
+  const Uint32 hostId= refToNode(aRef);
+  HostBuffer* const buffer = &hostBuffer[hostId];
+  const Uint32 TpacketLen= buffer->packetLenTA;
 
-  if ((Tlen + TpacketLen + 1) <= 25) {
-// ----------------------------------------------------------------
-// There is still space in the buffer. We will copy it into the
-// buffer.
-// ----------------------------------------------------------------
-    jam();
-    updatePackedList(signal, hostId);
-  } else if (false && TnoOfPackets == 1) {
-// ----------------------------------------------------------------
-// The buffer is full and there was only one packet buffered. We
-// will send this as a normal signal.
-// ----------------------------------------------------------------
-    Uint32 TnewRef= numberToRef((hostBuffer[hostId].packetBufferTA[0] >> 16),
-                                 hostId);
-    MEMCOPY_NO_WORDS(&signal->theData[0],
-                     &hostBuffer[hostId].packetBufferTA[1],
-                     TpacketLen - 1);
-    sendSignal(TnewRef, GSN_TRANSID_AI, signal, (TpacketLen - 1), JBB);
-    TpacketLen= 0;
-    TnoOfPackets= 0;
-  } else {
-// ----------------------------------------------------------------
-// The buffer is full but at least two packets. Send those in
-// packed form.
-// ----------------------------------------------------------------
-    MEMCOPY_NO_WORDS(&signal->theData[0],
-                     &hostBuffer[hostId].packetBufferTA[0],
-                     TpacketLen);
-    sendSignal(TBref, GSN_TRANSID_AI, signal, TpacketLen, JBB);
-    TpacketLen= 0;
-    TnoOfPackets= 0;
-  }
-// ----------------------------------------------------------------
-// Copy the signal into the buffer
-// ----------------------------------------------------------------
-  hostBuffer[hostId].packetBufferTA[TpacketLen + 0]= Theader;
-  hostBuffer[hostId].packetBufferTA[TpacketLen + 1]= sig0;
-  hostBuffer[hostId].packetBufferTA[TpacketLen + 2]= sig1;
-  hostBuffer[hostId].packetBufferTA[TpacketLen + 3]= sig2;
-  hostBuffer[hostId].noOfPacketsTA= TnoOfPackets + 1;
-  hostBuffer[hostId].packetLenTA= Tlen + TpacketLen + 1;
-  MEMCOPY_NO_WORDS(&hostBuffer[hostId].packetBufferTA[TpacketLen + 4],
-                   &signal->theData[25],
-                   Tlen - 3);
+  // ----------------------------------------------------------------
+  // There should always be space in the buffer.
+  // ----------------------------------------------------------------
+  ndbassert((TpacketLen + 1+TransIdAI::HeaderLength+lenOfData) <= 25);
+
+  // ----------------------------------------------------------------
+  // Copy the header + TRANSID_AI signal into the buffer
+  // ----------------------------------------------------------------
+  Uint32* const packedBuffer = &buffer->packetBufferTA[TpacketLen];
+  const Uint32 Theader= ((refToBlock(aRef) << 16)+lenOfData);
+  packedBuffer[0] = Theader;
+
+  MEMCOPY_NO_WORDS(&packedBuffer[1], signal->theData, TransIdAI::HeaderLength);
+  MEMCOPY_NO_WORDS(&packedBuffer[1+TransIdAI::HeaderLength], dataBuf, lenOfData);
+
+  buffer->packetLenTA= TpacketLen + 1+TransIdAI::HeaderLength+lenOfData;
+  buffer->noOfPacketsTA++;
+  updatePackedList(hostId);
 }
 
-void Dbtup::updatePackedList(Signal* signal, Uint16 hostId)
+void Dbtup::updatePackedList(Uint16 hostId)
 {
   if (hostBuffer[hostId].inPackedList == false) {
     Uint32 TpackedListIndex= cpackedListIndex;
-    jam();
+    jamDebug();
     hostBuffer[hostId].inPackedList= true;
     cpackedList[TpackedListIndex]= hostId;
     cpackedListIndex= TpackedListIndex + 1;
   }
 }
 
+/**
+ * Send a TRANSID_AI signal to an API node. If sufficiently small, the signal is 
+ * buffered for later being sent as a API_PACKED-signal. When required the 
+ * packed buffer is flushed to the destination API-node.
+ *
+ * Prereq:
+ *  - The destination node must be an API node.
+ *  - We must be connected to the API node.
+ */
+void Dbtup::sendAPI_TRANSID_AI(Signal* signal,
+                               Uint32 recBlockRef,
+                               const Uint32 *dataBuf,
+                               Uint32 lenOfData)
+{
+  const Uint32 nodeId= refToNode(recBlockRef);
+
+  // Test prerequisites:
+  ndbassert(getNodeInfo(nodeId).m_connected);
+  ndbassert(getNodeInfo(nodeId).m_type >= NodeInfo::API && getNodeInfo(nodeId).m_type <= NodeInfo::MGM);
+
+  ndbrequire(nodeId < MAX_NODES);
+  HostBuffer* const buffer = &hostBuffer[nodeId];
+  const Uint32 TpacketLen= buffer->packetLenTA;
+
+  /**
+   * Check if the packed buffers has to be flushed first.
+   * Note that even if we will not use them for this (too large) signal,
+   * it has to be flushed now in order to maintain the order of TRANSID_AIs
+   */
+  if (TpacketLen > 0 &&
+      TpacketLen + 1+TransIdAI::HeaderLength+lenOfData > 25)
+  {
+    jamDebug();
+    TransIdAI *transIdAI = (TransIdAI *)signal->getDataPtrSend();
+
+    // Save prepare TRANSID_AI header
+    const Uint32 sig0= transIdAI->connectPtr;
+    const Uint32 sig1= transIdAI->transId[0];
+    const Uint32 sig2= transIdAI->transId[1];
+
+    if (dataBuf != &signal->theData[25])
+    {
+      jamDebug();
+      /**
+       * TUP incorrectly guessed that it could prepare the signal
+       * to be EXECUTE_DIRECT'ly. Has to move it away for sendSignal()
+       * needing the low 25 signal-words to send the packed buffers.
+       * (Use memmove as src & dest may overlap)
+       */
+      memmove(&signal->theData[25], dataBuf, lenOfData*sizeof(Uint32));
+      dataBuf = &signal->theData[25];
+    }
+
+    // Send already buffered TRANSID_AI(s) preceeding this TRANSID_AI
+    const BlockReference TBref = numberToRef(API_PACKED, nodeId);
+    MEMCOPY_NO_WORDS(&signal->theData[0], &buffer->packetBufferTA[0], TpacketLen);
+    sendSignal(TBref, GSN_TRANSID_AI, signal, TpacketLen, JBB);
+    buffer->noOfPacketsTA = 0;
+    buffer->packetLenTA = 0;
+
+    // Reconstruct the current TRANSID_AI header
+    transIdAI->connectPtr = sig0;
+    transIdAI->transId[0] = sig1;
+    transIdAI->transId[1] = sig2;
+  }
+
+  if (lenOfData <= TransIdAI::DataLength)
+  {
+    /**
+     * Short signal, buffer it, or send directly
+     * 1) Buffer signal if we can pack at least
+     *    this signal + another 1-word signal into buffers.
+     * 2) else, short-signal is sent immediately.
+     *
+     * Note that the check for fitting a 1-word signal in addition
+     * to this signal serves dual purposes:
+     * - The 1-word signal is the smalles possible signal which
+     *   can either be added later, or already is buffered.
+     * - So failing to also add a 1-word signal implies that any
+     *   previously buffered signals were flushed above.
+     *   Thus, 'packetLenTA' is also known to be '== 0' in
+     *   the non-buffered sendSignal further below.
+     */
+#ifndef NDB_NO_DROPPED_SIGNAL
+    if (1+TransIdAI::HeaderLength + lenOfData +  // this TRANSID_AI
+        1+TransIdAI::HeaderLength + 1 <= 25)     // 1 word TRANSID_AI
+    {
+      jamDebug();
+      bufferTRANSID_AI(signal, recBlockRef, dataBuf, lenOfData);
+    }
+    else
+#endif
+    {
+      jamDebug();
+      ndbassert(buffer->packetLenTA == 0);
+      if (dataBuf != &signal->theData[TransIdAI::HeaderLength])
+      {
+        MEMCOPY_NO_WORDS(&signal->theData[TransIdAI::HeaderLength], dataBuf, lenOfData);
+      }
+      sendSignal(recBlockRef, GSN_TRANSID_AI, signal,
+                 TransIdAI::HeaderLength+lenOfData, JBB);
+    }
+  }
+  else
+  {
+    jamDebug();
+    /**
+     * Send to API as a long signal.
+     */
+    LinearSectionPtr ptr[3];
+    ptr[0].p= const_cast<Uint32*>(dataBuf);
+    ptr[0].sz= lenOfData;
+    sendSignal(recBlockRef, GSN_TRANSID_AI, signal,
+               TransIdAI::HeaderLength, JBB, ptr, 1);
+  }
+}
+
 /* ---------------------------------------------------------------- */
 /* ----------------------- SEND READ ATTRINFO --------------------- */
 /* ---------------------------------------------------------------- */
@@ -171,10 +263,8 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
 
   bool connectedToNode= getNodeInfo(nodeId).m_connected;
   const Uint32 type= getNodeInfo(nodeId).m_type;
-  bool is_api= (type >= NodeInfo::API && type <= NodeInfo::MGM);
-  bool old_dest= (getNodeInfo(nodeId).m_version < MAKE_VERSION(6,4,0));
-  Uint32 TpacketLen= hostBuffer[nodeId].packetLenTA;
-  Uint32 TpacketTA= hostBuffer[nodeId].noOfPacketsTA;
+  const bool is_api= (type >= NodeInfo::API && type <= NodeInfo::MGM);
+  //const bool old_dest= (getNodeInfo(nodeId).m_version < MAKE_VERSION(6,4,0));
 
   if (ERROR_INSERTED(4006) && (nodeId != getOwnNodeId())){
     // Use error insert to turn routing on
@@ -199,7 +289,7 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
    * check that there is no chance of sending TRANSID_AI_R to a block
    * that cannot handle it.
    */
-  ndbrequire(refToMain(routeBlockref) == DBTC || 
+  ndbassert (refToMain(routeBlockref) == DBTC || 
              /**
               * routeBlockref will point to SPJ for operations initiated by
               * that block. TRANSID_AI_R should not be sent to SPJ, as
@@ -234,85 +324,53 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
     /**
      * Own node -> execute direct
      */
-    if(nodeId != getOwnNodeId()){
-      jam();
+    if(nodeId != getOwnNodeId())
+    {
+      jamDebug();
     
-      /**
-       * Send long sig
-       */
-      if (ToutBufIndex >= 22 && is_api) {
-	jam();
-	/**
-	 * Flush buffer so that order is maintained
-	 */
-	if (TpacketTA != 0) {
-	  jam();
-	  BlockReference TBref = numberToRef(API_PACKED, nodeId);
-	  MEMCOPY_NO_WORDS(&signal->theData[0],
-			   &hostBuffer[nodeId].packetBufferTA[0],
-			   TpacketLen);
-	  sendSignal(TBref, GSN_TRANSID_AI, signal, TpacketLen, JBB);
-	  hostBuffer[nodeId].noOfPacketsTA = 0;
-	  hostBuffer[nodeId].packetLenTA = 0;
-	  transIdAI->connectPtr = sig0;
-	  transIdAI->transId[0] = sig1;
-	  transIdAI->transId[1] = sig2;
-	}//if
-	LinearSectionPtr ptr[3];
-	ptr[0].p= &signal->theData[25];
-	ptr[0].sz= ToutBufIndex;
-	sendSignal(recBlockref, GSN_TRANSID_AI, signal, 3, JBB, ptr, 1);
-	return;
-      }
-      
-      /**
-       * Send long signal to DBUTIL.
-       */
-      const Uint32 block= refToMain(recBlockref);
-      if ((block == DBUTIL || block == DBSPJ) && !old_dest) {
-	jam();
-	LinearSectionPtr ptr[3];
-	ptr[0].p= &signal->theData[25];
-	ptr[0].sz= ToutBufIndex;
-	sendSignal(recBlockref, GSN_TRANSID_AI, signal, 3, JBB, ptr, 1);
-	return;
+      //Assume no prehistoric versions not supporting long TRANSID_AI signals anymore
+      ndbassert(getNodeInfo(nodeId).m_version >= MAKE_VERSION(6,4,0));
+
+      if (is_api)
+      {
+        sendAPI_TRANSID_AI(signal, recBlockref,
+                           &signal->theData[25], ToutBufIndex);
       }
 
       /**
-       * short sig + api -> buffer
+       * Send long signal if 'long' data.
+       * Note that SPJ can *only* handle long signals
        */
-#ifndef NDB_NO_DROPPED_SIGNAL
-      if (ToutBufIndex < 22 && is_api){
-	jam();
-	bufferTRANSID_AI(signal, recBlockref, 3+ToutBufIndex);
-	return;
+      else if (ToutBufIndex > TransIdAI::DataLength ||
+               refToMain(recBlockref) == DBSPJ)
+      {
+        jam();
+        /**
+         * Receiver block doesn't support packed 'short' signals.
+         */
+        LinearSectionPtr ptr[3];
+        ptr[0].p= &signal->theData[25];
+        ptr[0].sz= ToutBufIndex;
+        sendSignal(recBlockref, GSN_TRANSID_AI, signal,
+                   TransIdAI::HeaderLength, JBB, ptr, 1);
       }
-#endif      
-
-      /**
-       * rest -> old send sig
-       */
-      Uint32 * src= signal->theData+25;
-      if (ToutBufIndex >= 22){
-	do {
-	  jam();
-	  MEMCOPY_NO_WORDS(&signal->theData[3], src, 22);
-	  sendSignal(recBlockref, GSN_TRANSID_AI, signal, 25, JBB);
-	  ToutBufIndex -= 22;
-	  src += 22;
-	} while(ToutBufIndex >= 22);
-      }
-      
-      if (ToutBufIndex > 0){
-	jam();
-	MEMCOPY_NO_WORDS(&signal->theData[3], src, ToutBufIndex);
-	sendSignal(recBlockref, GSN_TRANSID_AI, signal, 3+ToutBufIndex, JBB);
+      else
+      {
+        jam();
+        ndbassert(ToutBufIndex <= TransIdAI::DataLength);
+        /**
+         * Data is 'short', send short signal
+         */
+        MEMCOPY_NO_WORDS(&signal->theData[TransIdAI::HeaderLength],
+                         &signal->theData[25], ToutBufIndex);
+        sendSignal(recBlockref, GSN_TRANSID_AI, signal,
+                   TransIdAI::HeaderLength+ToutBufIndex, JBB);
       }
       return;
-    }
-
+    } //nodeId != getOwnNodeId()
+  
     /**
-     * BACKUP/LQH run in our thread, so we can EXECUTE_DIRECT().
+     * BACKUP, LQH & SUMA run in our thread, so we can EXECUTE_DIRECT().
      *
      * The UTIL/TC blocks are in another thread (in multi-threaded ndbd), so
      * must use sendSignal().
@@ -327,14 +385,15 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
          blockNumber == DBLQH ||
          blockNumber == SUMA))
     {
-      EXECUTE_DIRECT(blockNumber, GSN_TRANSID_AI, signal, 3 + ToutBufIndex);
-      jamEntry();
+      EXECUTE_DIRECT(blockNumber, GSN_TRANSID_AI, signal,
+                     TransIdAI::HeaderLength + ToutBufIndex);
+      jamEntryDebug();
     }
     else
     {
       jam();
       LinearSectionPtr ptr[3];
-      ptr[0].p= &signal->theData[3];
+      ptr[0].p= &signal->theData[TransIdAI::HeaderLength];
       ptr[0].sz= ToutBufIndex;
       if (ERROR_INSERTED(4038))
       {
@@ -343,7 +402,8 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
         ndbrequire(appendToSection(sectionIVal, ptr[0].p, ptr[0].sz));
         SectionHandle sh(this, sectionIVal);
         
-        sendSignalWithDelay(recBlockref, GSN_TRANSID_AI, signal, 10, 3, &sh);
+        sendSignalWithDelay(recBlockref, GSN_TRANSID_AI, signal, 10,
+                            TransIdAI::HeaderLength, &sh);
       }
       else
       {
@@ -362,7 +422,7 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
         sendSignal(recBlockref,
                    GSN_TRANSID_AI,
                    signal,
-                   3,
+                   TransIdAI::HeaderLength,
                    prioLevel,
                    ptr,
                    1);
@@ -392,5 +452,6 @@ void Dbtup::sendReadAttrinfo(Signal* signal,
   LinearSectionPtr ptr[3];
   ptr[0].p= &signal->theData[25];
   ptr[0].sz= ToutBufIndex;
-  sendSignal(routeBlockref, GSN_TRANSID_AI_R, signal, 4, JBB, ptr, 1);
+  sendSignal(routeBlockref, GSN_TRANSID_AI_R, signal,
+             TransIdAI::HeaderLength+1, JBB, ptr, 1);
 }
