@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2006, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2006, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,6 +25,8 @@
 #ifndef RWPOOL_HPP
 #define RWPOOL_HPP
 
+#include "ndbd_exit_codes.h"
+#include "NdbOut.hpp"
 #include "Pool.hpp"
 
 #define JAM_FILE_ID 311
@@ -45,6 +47,7 @@ struct RWPage
 /**
  * Read Write  Pool
  */
+template<typename T>
 struct RWPool
 {
   Record_info m_record_info;
@@ -57,24 +60,26 @@ struct RWPool
   Uint16 m_current_first_free;
   Uint16 m_current_ref_count;
 public:
+  typedef T Type;
   RWPool();
   
   void init(const Record_info& ri, const Pool_context& pc);
-  bool seize(Ptr<void>&);
-  void release(Ptr<void>);
-  void * getPtr(Uint32 i);
-  void * getPtr(const Record_info&ri, Uint32 i);
+  bool seize(Ptr<T>&);
+  void release(Ptr<T>);
+  void * getPtr(Uint32 i) const;
+  void * getPtr(const Record_info&ri, Uint32 i) const;
   
   STATIC_CONST( WORDS_PER_PAGE = RWPage::RWPAGE_WORDS );
 
 private:  
-  void handle_invalid_release(Ptr<void>) ATTRIBUTE_NORETURN;
-  void handle_invalid_get_ptr(Uint32 i) ATTRIBUTE_NORETURN;
+  void handle_invalid_release(Ptr<T>) ATTRIBUTE_NORETURN;
+  void handle_invalid_get_ptr(Uint32 i) const ATTRIBUTE_NORETURN;
 };
 
+template<typename T>
 inline
 void*
-RWPool::getPtr(Uint32 i)
+RWPool<T>::getPtr(Uint32 i) const
 {
   Uint32 page_no = i >> POOL_RECORD_BITS;
   Uint32 page_idx = i & POOL_RECORD_MASK;
@@ -89,9 +94,10 @@ RWPool::getPtr(Uint32 i)
   return 0;                                     /* purify: deadcode */
 }
 
+template<typename T>
 inline
 void*
-RWPool::getPtr(const Record_info &ri, Uint32 i)
+RWPool<T>::getPtr(const Record_info &ri, Uint32 i) const
 {
   Uint32 page_no = i >> POOL_RECORD_BITS;
   Uint32 page_idx = i & POOL_RECORD_MASK;
@@ -106,6 +112,7 @@ RWPool::getPtr(const Record_info &ri, Uint32 i)
   return 0;                                     /* purify: deadcode */
 }
 
+#include "RWPool.cpp"
 
 #undef JAM_FILE_ID
 
