@@ -594,12 +594,8 @@ SegmentSubPool::checkInvariants()
 
 #ifdef TEST_SEGMENTLIST
 
-#undef JAM_FILE_ID
-
 #include <NdbTap.hpp>
 
-#define JAM_FILE_ID 494
-
 /* Redefine ArrayPool dependencies to enable standalone Unit-test compile */
 void ErrorReporter::handleAssert(const char* message, const char* file, int line, int ec)
 {
@@ -616,7 +612,7 @@ SectionSegmentPool g_sectionSegmentPool;
 
 /* Static function */
 void
-SectionSegmentPool::handleOutOfSegments(ArrayPool<SectionSegment>& pool)
+SectionSegmentPool::handleOutOfSegments(SectionSegment_basepool& pool)
 {
   printf("SectionSegmentPool::handleOutOfSegments called");
 }
