@@ -33,12 +33,31 @@
 
 #define JAM_FILE_ID 282
 
+/**
+ * To enable jamDebug and its siblings in a production simply
+ * remove the comment and get EXTRA_JAM defined.
+ * It is enabled in builds using ERROR_INSERT to simplify tracing
+ * of bugs from autotest.
+ *
+ * Similarly enable initialisation of global variables in a block
+ * thread before executing each asynchronous signal by enabling
+ * USE_INIT_GLOBAL_VARIABLES. This is also enabled in all builds
+ * using ERROR_INSERT to ensure that we quickly discover failures
+ * in using global variables.
+ */
+#if defined(ERROR_INSERT)
+#define EXTRA_JAM 1
+#define USE_INIT_GLOBAL_VARIABLES 1
+#endif
 
 #ifdef NO_EMULATED_JAM
 
 #define jam()
 #define jamLine(line)
 #define jamEntry()
+#define jamDebug()
+#define jamLineDebug(line)
+#define jamEntryDebug()
 #define jamEntryLine(line)
 #define jamBlock(block)
 #define jamBlockLine(block, line)
@@ -50,6 +69,10 @@
 #define thrjamEntryLine(buf, line)
 #define thrjam(buf)
 #define thrjamLine(buf, line)
+#define thrjamEntryDebug(buf)
+#define thrjamEntryLineDebug(buf, line)
+#define thrjamDebug(buf)
+#define thrjamLineDebug(buf, line)
 
 #else
 
@@ -95,6 +118,23 @@
 #define thrjam(buf) thrjamLine(buf, __LINE__)
 #define thrjamEntry(buf) thrjamEntryLine(buf, __LINE__)
 
+#if defined VM_TRACE || defined ERROR_INSERT || defined EXTRA_JAM
+#define jamDebug() jam()
+#define jamLineDebug(line) jamLine(line)
+#define jamEntryDebug() jamEntry()
+#define thrjamEntryDebug(buf) thrjamEntry(buf)
+#define thrjamEntryLineDebug(buf, line) thrJamEntryLine(guf, line)
+#define thrjamDebug(buf) thrjam(buf)
+#define thrjamLineDebug(buf, line) thrjamLine(buf, line)
+#else
+#define jamDebug()
+#define jamLineDebug(line)
+#define jamEntryDebug()
+#define thrjamEntryDebug(buf)
+#define thrjamEntryLineDebug(buf, line)
+#define thrjamDebug(buf)
+#define thrjamLineDebug(buf, line)
+#endif
 #endif
 
 #ifndef NDB_OPT
@@ -210,6 +250,7 @@
 #define NO_LCP
 #define NO_GCP
 #endif
+#define ZUNDEFINED_GCI_LIMIT 1
 
 /**
  * Ndb kernel blocks assertion handling
