@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,18 +35,26 @@
 #include <NdbSleep.h>
 #include <NdbOut.hpp>
 #include <NdbTick.h>
-
+#include <ProcessInfo.hpp>
+#include <OwnProcessInfo.hpp>
 
 #include <signaldata/NodeFailRep.hpp>
 #include <signaldata/NFCompleteRep.hpp>
 #include <signaldata/ApiRegSignalData.hpp>
 #include <signaldata/AlterTable.hpp>
 #include <signaldata/SumaImpl.hpp>
+#include <signaldata/ProcessInfoRep.hpp>
 
 #include <mgmapi.h>
 #include <mgmapi_configuration.hpp>
 #include <mgmapi_config_parameters.h>
 
+#if 0
+#define DEBUG_FPRINTF(arglist) do { fprintf arglist ; } while (0)
+#else
+#define DEBUG_FPRINTF(a)
+#endif
+
 int global_flag_skip_invalidate_cache = 0;
 int global_flag_skip_waiting_for_clean_cache = 0;
 //#define DEBUG_REG
@@ -73,6 +81,7 @@ ClusterMgr::ClusterMgr(TransporterFacade & _facade):
   noOfConnectedDBNodes(0),
   minDbVersion(0),
   theClusterMgrThread(NULL),
+  m_process_info(NULL),
   m_cluster_state(CS_waiting_for_clean_cache),
   m_hbFrequency(0)
 {
@@ -101,6 +110,7 @@ ClusterMgr::~ClusterMgr()
   }
   NdbCondition_Destroy(waitForHBCond);
   NdbMutex_Destroy(clusterMgrThreadMutex);
+  ProcessInfo::release(m_process_info);
   DBUG_VOID_RETURN;
 }
 
@@ -202,11 +212,14 @@ ClusterMgr::configure(Uint32 nodeId,
 
   theFacade.get_registry()->set_connect_backoff_max_time_in_ms(
     start_connect_backoff_max_time);
+
+  m_process_info = ProcessInfo::forNodeId(nodeId);
 }
 
 void
 ClusterMgr::startThread()
 {
+  DBUG_ENTER("ClusterMgr::startThread");
   /**
    * We use the clusterMgrThreadMutex as a signalling object between this
    * thread and the main thread of the ClusterMgr.
@@ -220,6 +233,13 @@ ClusterMgr::startThread()
                                          0, // default stack size
                                          "ndb_clustermgr",
                                          NDB_THREAD_PRIO_HIGH);
+  if (theClusterMgrThread == NULL)
+  {
+    ndbout_c("ClusterMgr::startThread: Failed to create thread for cluster management.");
+    assert(theClusterMgrThread != NULL);
+    DBUG_VOID_RETURN;
+  }
+
   Uint32 cnt = 0;
   while (theStop == -1 && cnt < 60)
   {
@@ -227,6 +247,7 @@ ClusterMgr::startThread()
   }
 
   assert(theStop == 0);
+  DBUG_VOID_RETURN;
 }
 
 void
@@ -282,7 +303,7 @@ ClusterMgr::startup()
   for (Uint32 i = 0; i<3000; i++)
   {
     theFacade.request_connection_check();
-    start_poll();
+    prepare_poll();
     do_poll(0);
     complete_poll();
 
@@ -331,7 +352,7 @@ ClusterMgr::threadMain()
       NdbSleep_MilliSleep(minHeartBeatInterval/5);
       {
         /**
-         * start_poll does lock the trp_client and complete_poll
+         * prepare_poll does lock the trp_client and complete_poll
          * releases this lock. This means that this protects
          * against concurrent calls to send signals in ArbitMgr.
          * We do however need to protect also against concurrent
@@ -340,7 +361,7 @@ ClusterMgr::threadMain()
          * poll.
          */
         Guard g(clusterMgrThreadMutex);
-        start_poll();
+        prepare_poll();
         do_poll(0);
         complete_poll();
       }
@@ -545,6 +566,16 @@ ClusterMgr::trp_deliver_signal(const NdbApiSignal* sig,
       // SUB_GCP_COMPLETE_ACK will be sent per node per epoch, with
       // minimum interval of TimeBetweenEpochs.
       safe_sendSignal(&tSignal, aNodeId);
+
+      /**
+       * Note:
+       * After fixing #Bug#22705935 'sendSignal() flush optimization isses',
+       * we could likely just as well have used safe_noflush_sendSignal() above.
+       * (and several other places)
+       * That patch ensures that any buffered signals sent while 
+       * delivering signals are flushed as soon as we have processed the 
+       * chunk of signals to be delivered.
+       */ 
     }
     break;
   }
@@ -556,16 +587,6 @@ ClusterMgr::trp_deliver_signal(const NdbApiSignal* sig,
     theFacade.for_each(this, sig, ptr);
     return;
   }
-  case GSN_CONNECT_REP:
-  {
-    execCONNECT_REP(sig, ptr);
-    return;
-  }
-  case GSN_DISCONNECT_REP:
-  {
-    execDISCONNECT_REP(sig, ptr);
-    return;
-  }
   case GSN_CLOSE_COMREQ:
   {
     theFacade.perform_close_clnt(this);
@@ -584,7 +605,7 @@ ClusterMgr::trp_deliver_signal(const NdbApiSignal* sig,
 }
 
 ClusterMgr::Node::Node()
-  : hbFrequency(0), hbCounter(0)
+  : hbFrequency(0), hbCounter(0), processInfoSent(0)
 {
 }
 
@@ -653,6 +674,45 @@ ClusterMgr::recalcMinDbVersion()
   minDbVersion = newMinDbVersion;
 }
 
+/******************************************************************************
+ * Send PROCESSINFO_REP
+ ******************************************************************************/
+void
+ClusterMgr::sendProcessInfoReport(NodeId nodeId)
+{
+  LinearSectionPtr ptr[3];
+  LinearSectionPtr & pathSection = ptr[ProcessInfoRep::PathSectionNum];
+  LinearSectionPtr & hostSection = ptr[ProcessInfoRep::HostSectionNum];
+  BlockReference ownRef = numberToRef(API_CLUSTERMGR, theFacade.ownId());
+  NdbApiSignal signal(ownRef);
+  int nsections = 0;
+  signal.theVerId_signalNumber = GSN_PROCESSINFO_REP;
+  signal.theReceiversBlockNumber = QMGR;
+  signal.theTrace  = 0;
+  signal.theLength = ProcessInfoRep::SignalLength;
+
+  ProcessInfoRep * report = CAST_PTR(ProcessInfoRep, signal.getDataPtrSend());
+  m_process_info->buildProcessInfoReport(report);
+
+  const char * uri_path = m_process_info->getUriPath();
+  pathSection.p = (Uint32 *) uri_path;
+  pathSection.sz = ProcessInfo::UriPathLengthInWords;
+  if(uri_path[0])
+  {
+    nsections = 1;
+  }
+
+  const char * hostAddress = m_process_info->getHostAddress();
+  if(hostAddress[0])
+  {
+    nsections = 2;
+    hostSection.p = (Uint32 *) hostAddress;
+    hostSection.sz = ProcessInfo::AddressStringLengthInWords;
+  }
+  safe_noflush_sendSignal(&signal, nodeId, ptr, nsections);
+}
+
+
 /******************************************************************************
  * API_REGREQ and friends
  ******************************************************************************/
@@ -830,6 +890,15 @@ ClusterMgr::execAPI_REGCONF(const NdbApiSignal * signal,
     }
   }
 
+  /* Send ProcessInfo Report to a newly connected DB node */
+  if ( cm_node.m_info.m_type == NodeInfo::DB &&
+       ndbd_supports_processinfo(cm_node.m_info.m_version) &&
+       (! cm_node.processInfoSent) )
+  {
+    sendProcessInfoReport(nodeId);
+    cm_node.processInfoSent = true;
+  }
+
   // Distribute signal to all threads/blocks
   // TODO only if state changed...
   theFacade.for_each(this, signal, ptr);
@@ -885,19 +954,26 @@ ClusterMgr::execNF_COMPLETEREP(const NdbApiSignal* signal,
 }
 
 /**
- * This is called as a callback when executing update_connections which
- * is always called with ownership of trp_client lock.
+ * ::reportConnected() and ::reportDisconnected()
+ *
+ * Should be called from the client thread being the poll owner,
+ * which could either be ClusterMgr itself, or another API client.
+ *
+ * As ClusterMgr maintains shared global data, updating
+ * its connection state needs m_mutex being locked.
+ * If ClusterMgr is the poll owner, it already owns that
+ * lock, else it has to be locked now.
  */
 void
 ClusterMgr::reportConnected(NodeId nodeId)
 {
   DBUG_ENTER("ClusterMgr::reportConnected");
   DBUG_PRINT("info", ("nodeId: %u", nodeId));
-  /**
-   * Ensure that we are sending heartbeat every 100 ms
-   * until we have got the first reply from NDB providing
-   * us with the real time-out period to use.
-   */
+  assert(theFacade.is_poll_owner_thread());
+
+  if (theFacade.m_poll_owner != this)
+    lock();
+
   assert(nodeId > 0 && nodeId < MAX_NODES);
   if (nodeId != getOwnNodeId())
   {
@@ -917,9 +993,15 @@ ClusterMgr::reportConnected(NodeId nodeId)
     }
   }
 
+  /**
+   * Ensure that we are sending heartbeat every 100 ms
+   * until we have got the first reply from NDB providing
+   * us with the real time-out period to use.
+   */
   cm_node.hbMissed = 0;
   cm_node.hbCounter = 0;
   cm_node.hbFrequency = 0;
+  cm_node.processInfoSent = false;
 
   assert(theNode.is_connected() == false);
 
@@ -927,6 +1009,8 @@ ClusterMgr::reportConnected(NodeId nodeId)
    * make sure the node itself is marked connected even
    * if first API_REGCONF has not arrived
    */
+  DEBUG_FPRINTF((stderr, "(%u)theNode.set_connected(true) for node: %u\n",
+                         getOwnNodeId(), nodeId));
   theNode.set_connected(true);
   theNode.m_state.m_connected_nodes.set(nodeId);
   theNode.m_info.m_version = 0;
@@ -935,13 +1019,17 @@ ClusterMgr::reportConnected(NodeId nodeId)
   theNode.m_node_fail_rep = false;
   theNode.m_state.startLevel = NodeState::SL_NOTHING;
   theNode.minDbVersion = 0;
+
+  /**
+   * End of protected ClusterMgr updates of shared global data.
+   * Informing other API client does not need a global protection
+   */ 
+  if (theFacade.m_poll_owner != this)
+    unlock();
   
   /**
-   * We know that we have clusterMgrThreadMutex and trp_client::mutex
-   *   but we don't know if we are polling...and for_each can
-   *   only be used by a poller...
-   *
-   * Send signal to self, so that we can do this when receiving a signal
+   * We are called by the poll owner (asserted above), so we can
+   * tell each API client about the CONNECT_REP ourself.
    */
   NdbApiSignal signal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
   signal.theVerId_signalNumber = GSN_CONNECT_REP;
@@ -949,74 +1037,38 @@ ClusterMgr::reportConnected(NodeId nodeId)
   signal.theTrace  = 0;
   signal.theLength = 1;
   signal.getDataPtrSend()[0] = nodeId;
-  raw_sendSignal(&signal, getOwnNodeId());
+  theFacade.for_each(this, &signal, NULL);
   DBUG_VOID_RETURN;
 }
 
-void
-ClusterMgr::execCONNECT_REP(const NdbApiSignal* sig,
-                            const LinearSectionPtr ptr[])
-{
-  theFacade.for_each(this, sig, 0);
-}
-
-void
-ClusterMgr::set_node_dead(trp_node& theNode)
-{
-  set_node_alive(theNode, false);
-  theNode.set_confirmed(false);
-  theNode.m_state.m_connected_nodes.clear();
-  theNode.m_state.startLevel = NodeState::SL_NOTHING;
-  theNode.m_info.m_connectCount ++;
-  theNode.nfCompleteRep = false;
-}
-
 void
 ClusterMgr::reportDisconnected(NodeId nodeId)
 {
+  assert(theFacade.is_poll_owner_thread());
   assert(nodeId > 0 && nodeId < MAX_NODES);
 
-  /**
-   * We know that we have trp_client lock
-   *   but we don't know if we are polling...and for_each can
-   *   only be used by a poller...
-   *
-   * Send signal to self, so that we can do this when receiving a signal
-   */
-  NdbApiSignal signal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
-  signal.theVerId_signalNumber = GSN_DISCONNECT_REP;
-  signal.theReceiversBlockNumber = API_CLUSTERMGR;
-  signal.theTrace  = 0;
-  signal.theLength = DisconnectRep::SignalLength;
+  if (theFacade.m_poll_owner != this)
+    lock();
 
-  DisconnectRep * rep = CAST_PTR(DisconnectRep, signal.getDataPtrSend());
-  rep->nodeId = nodeId;
-  rep->err = 0;
-  raw_sendSignal(&signal, getOwnNodeId());
-}
-
-void
-ClusterMgr::execDISCONNECT_REP(const NdbApiSignal* sig,
-                               const LinearSectionPtr ptr[])
-{
-  const DisconnectRep * rep = CAST_CONSTPTR(DisconnectRep, sig->getDataPtr());
-  Uint32 nodeId = rep->nodeId;
-
-  assert(nodeId > 0 && nodeId < MAX_NODES);
   Node & cm_node = theNodes[nodeId];
   trp_node & theNode = cm_node;
 
-  bool node_failrep = theNode.m_node_fail_rep;
-  bool node_connected = theNode.is_connected();
+  const bool node_failrep = theNode.m_node_fail_rep;
+  const bool node_connected = theNode.is_connected();
   set_node_dead(theNode);
+  DEBUG_FPRINTF((stderr, "(%u)theNode.set_connected(false) for node: %u\n",
+                         getOwnNodeId(), nodeId));
   theNode.set_connected(false);
 
   /**
    * Remaining processing should only be done if the node
    * actually completed connecting...
    */
-  if (!node_connected)
+  if (unlikely(!node_connected))
   {
+    assert(node_connected);
+    if (theFacade.m_poll_owner != this)
+      unlock();
     return;
   }
 
@@ -1052,10 +1104,20 @@ ClusterMgr::execDISCONNECT_REP(const NdbApiSignal* sig,
     }
   }
 
+  /**
+   * End of protected ClusterMgr updates of shared global data.
+   * Informing other API client does not need a global protection
+   */
+  if (theFacade.m_poll_owner != this)
+    unlock();
+
   if (node_failrep == false)
   {
     /**
      * Inform API
+     *
+     * We are called by the poll owner (asserted above), so we can
+     * tell each API client about the NODE_FAILREP ourself.
      */
     NdbApiSignal signal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
     signal.theVerId_signalNumber = GSN_NODE_FAILREP;
@@ -1155,6 +1217,17 @@ ClusterMgr::execNODE_FAILREP(const NdbApiSignal* sig,
   }
 }
 
+void
+ClusterMgr::set_node_dead(trp_node& theNode)
+{
+  set_node_alive(theNode, false);
+  theNode.set_confirmed(false);
+  theNode.m_state.m_connected_nodes.clear();
+  theNode.m_state.startLevel = NodeState::SL_NOTHING;
+  theNode.m_info.m_connectCount ++;
+  theNode.nfCompleteRep = false;
+}
+
 bool
 ClusterMgr::is_cluster_completely_unavailable()
 {
@@ -1242,6 +1315,24 @@ ClusterMgr::print_nodes(const char* where, NdbOut& out)
   out << "<<" << endl;
 }
 
+void
+ClusterMgr::setProcessInfoUri(const char * scheme, const char * address_string,
+                              int port, const char * path)
+{
+  Guard g(clusterMgrThreadMutex);
+
+  m_process_info->setUriScheme(scheme);
+  m_process_info->setHostAddress(address_string);
+  m_process_info->setPort(port);
+  m_process_info->setUriPath(path);
+
+  /* Set flag to resend ProcessInfo Report */
+  for(int i = 1; i < MAX_NODES ; i++)
+  {
+    Node & node = theNodes[i];
+    if(node.is_connected()) node.processInfoSent = false;
+  }
+}
 
 /******************************************************************************
  * Arbitrator
@@ -1287,6 +1378,7 @@ ArbitMgr::~ArbitMgr()
 void
 ArbitMgr::doStart(const Uint32* theData)
 {
+  DBUG_ENTER("ArbitMgr::doStart");
   ArbitSignal aSignal;
   NdbMutex_Lock(theThreadMutex);
   if (theThread != NULL) {
@@ -1306,7 +1398,13 @@ ArbitMgr::doStart(const Uint32* theData)
     0, // default stack size
     "ndb_arbitmgr",
     NDB_THREAD_PRIO_HIGH);
+  if (theThread == NULL)
+  {
+    ndbout_c("ArbitMgr::doStart: Failed to create thread for arbitration.");
+    assert(theThread != NULL);
+  }
   NdbMutex_Unlock(theThreadMutex);
+  DBUG_VOID_RETURN;
 }
 
 // The "choose me" signal from a candidate.
