@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -82,7 +82,7 @@ HugoQueryBuilder::fixOptions()
 }
 
 void
-HugoQueryBuilder::addTable(Ndb* ndb, const NdbDictionary::Table* tab)
+HugoQueryBuilder::addTable(const NdbDictionary::Table* tab)
 {
   for (unsigned i = 0; i<m_tables.size(); i++)
   {
@@ -93,7 +93,7 @@ HugoQueryBuilder::addTable(Ndb* ndb, const NdbDictionary::Table* tab)
   TableDef def;
   def.m_table = tab;
 
-  NdbDictionary::Dictionary* pDict = ndb->getDictionary();
+  NdbDictionary::Dictionary* pDict = m_ndb->getDictionary();
   NdbDictionary::Dictionary::List l;
   int res = pDict->listIndexes(l, tab->getName());
   if (res == 0)
@@ -494,6 +494,11 @@ loop:
       // We have grandparents, 'parents[0]' is real parent
       options.setParent(parents[0].m_op);
     }
+    if ((rand() % 2) == 0)
+    {
+      // Set INNER-join options (no NULL extended rows returned)
+      options.setMatchType(NdbQueryOptions::MatchNonNull);
+    }
     switch(oi.m_type){
     case NdbQueryOperationDef::PrimaryKeyAccess:{
       int opNo = 0;
@@ -618,7 +623,7 @@ HugoQueryBuilder::createQuery(bool takeOwnership)
 
   m_options = save;
 
-  const NdbQueryDef * def = builder->prepare();
+  const NdbQueryDef * def = builder->prepare(m_ndb);
   builder->destroy();
   if (def != 0 && !takeOwnership)
   {
