@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -26,17 +33,23 @@
 #include <ndb_opts.h>
 
 #include <NdbApi.hpp>
-#include <NDBT.hpp>
+#include <NdbOut.hpp>
+
+#include <NdbToolsProgramExitCodes.hpp>
 
 static Ndb_cluster_connection *ndb_cluster_connection= 0;
 static Ndb* ndb = 0;
 static const NdbDictionary::Dictionary * dic = 0;
-static int _unqualified = 0;
+static int _fully_qualified = 0;
 static int _parsable = 0;
 static int show_temp_status = 0;
 
 const char *load_default_groups[]= { "mysql_cluster",0 };
 
+static void
+fatal(char const* fmt, ...)
+  ATTRIBUTE_FORMAT(printf, 1, 2);
+
 static void
 fatal(char const* fmt, ...)
 {
@@ -49,10 +62,13 @@ fatal(char const* fmt, ...)
     if (ndb)
       ndbout << " - " << ndb->getNdbError();
     ndbout << endl;
-    NDBT_ProgramExit(NDBT_FAILED);
-    exit(1);
+    exit(NdbToolsProgramExitCode::FAILED);
 }
 
+static void
+fatal_dict(char const* fmt, ...)
+  ATTRIBUTE_FORMAT(printf, 1, 2);
+
 static void
 fatal_dict(char const* fmt, ...)
 {
@@ -65,20 +81,31 @@ fatal_dict(char const* fmt, ...)
     if (dic)
       ndbout << " - " << dic->getNdbError();
     ndbout << endl;
-    NDBT_ProgramExit(NDBT_FAILED);
-    exit(1);
+    exit(NdbToolsProgramExitCode::FAILED);
 }
 
 static void
-list(const char * tabname, 
+list(const char * tabname,
      NdbDictionary::Object::Type type)
 {
+    /**
+     * Display fully qualified table names if --fully-qualified is set to 1.
+     *
+     * useFq passed to listObjects() and listIndexes() below in this context
+     * actually behaves like 'unqualified'.
+     * useFq == true : Strip off the database and schema (and tableid) and
+     * return the table/index name
+     * useFq == false : Return the full name
+     * (database/schema/[tableid/]indexname|tablename)
+     */
+    bool useFq = !_fully_qualified;
+
     NdbDictionary::Dictionary::List list;
     if (tabname == 0) {
-	if (dic->listObjects(list, type) == -1)
+	if (dic->listObjects(list, type, useFq) == -1)
 	    fatal_dict("listObjects");
     } else {
-	if (dic->listIndexes(list, tabname) == -1)
+	if (dic->listIndexes(list, tabname, useFq) == -1)
 	    fatal_dict("listIndexes");
     }
     if (!_parsable)
@@ -149,6 +176,21 @@ list(const char * tabname,
         case NdbDictionary::Object::TableEvent:
             strcpy(type, "TableEvent");
             break;
+        case NdbDictionary::Object::ForeignKey:
+            strcpy(type, "ForeignKey");
+            break;
+        case NdbDictionary::Object::FKParentTrigger:
+            strcpy(type, "FKParentTrigger");
+            break;
+        case NdbDictionary::Object::FKChildTrigger:
+            strcpy(type, "FKChildTrigger");
+            break;
+        case NdbDictionary::Object::HashMap:
+            strcpy(type, "HashMap");
+            break;
+        case NdbDictionary::Object::FullyReplicatedTrigger:
+            strcpy(type, "FullyRepTrigger");
+            break;
         default:
 	  sprintf(type, "%d", (int)elt.type);
             break;
@@ -248,7 +290,7 @@ list(const char * tabname,
         }
     }
     if (_parsable)
-      exit(0);
+      exit(NdbToolsProgramExitCode::OK);
 }
 
 static const char* _dbname = 0;
@@ -268,9 +310,9 @@ static struct my_option my_long_options[] =
   { "type", 't', "type",
     (uchar**) &_type, (uchar**) &_type, 0,
     GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 }, 
-  { "unqualified", 'u', "Use unqualified table names",
-    (uchar**) &_unqualified, (uchar**) &_unqualified, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 }, 
+  { "fully-qualified", 'f', "Show fully qualified table names",
+    (uchar**) &_fully_qualified, (uchar**) &_fully_qualified, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "parsable", 'p', "Return output suitable for mysql LOAD DATA INFILE",
     (uchar**) &_parsable, (uchar**) &_parsable, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 }, 
@@ -295,22 +337,31 @@ int main(int argc, char** argv){
   ndb_opt_set_usage_funcs(short_usage_sub, usage);
   ndb_load_defaults(NULL,load_default_groups,&argc,&argv);
   int ho_error;
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   opt_debug= "d:t:O,/tmp/ndb_show_tables.trace";
 #endif
-  if ((ho_error=handle_options(&argc, &argv, my_long_options,
-			       ndb_std_get_one_option)))
-    return NDBT_ProgramExit(NDBT_WRONGARGS);
-  if(_dbname && argc==0) {
+  bool using_default_database = false;
+  if ((ho_error = handle_options(&argc, &argv, my_long_options,
+                                 ndb_std_get_one_option)))
+  {
+    ndb_free_defaults(argv);
+    return NdbToolsProgramExitCode::WRONG_ARGS;
+  }
+
+  if(_dbname && argc == 0)
+  {
     ndbout << "-d option given without table name." << endl;
-    return NDBT_ProgramExit(NDBT_WRONGARGS);
+    ndb_free_defaults(argv);
+    return NdbToolsProgramExitCode::WRONG_ARGS;
   }
   if (argc>0)
       _tabname = argv[0];
-  if (argc > 1) {
+  if (argc > 1)
+  {
     ndbout << "Wrong Argument" << endl;
     ndbout << "Please use the option --help for usage." << endl;
-    return NDBT_ProgramExit(NDBT_WRONGARGS);
+    ndb_free_defaults(argv);
+    return NdbToolsProgramExitCode::WRONG_ARGS;
   }
 
   ndb_cluster_connection = new Ndb_cluster_connection(opt_ndb_connectstring,
@@ -319,7 +370,7 @@ int main(int argc, char** argv){
     fatal("Unable to create cluster connection");
 
   ndb_cluster_connection->set_name("ndb_show_tables");
-  if (ndb_cluster_connection->connect(12,5,1))
+  if (ndb_cluster_connection->connect(opt_connect_retries - 1, opt_connect_retry_delay, 1))
     fatal("Unable to connect to management server.\n - Error: '%d: %s'",
           ndb_cluster_connection->get_latest_error(),
           ndb_cluster_connection->get_latest_error_msg());
@@ -330,13 +381,26 @@ int main(int argc, char** argv){
   if (ndb->init() != 0)
     fatal("init");
   if (_dbname == 0 && _tabname != 0)
+  {
     _dbname = "TEST_DB";
+    using_default_database = true;
+  }
   ndb->setDatabaseName(_dbname);
   dic = ndb->getDictionary();
   if( argc >0){
     if(!dic->getTable(_tabname)){
-      ndbout << _tabname << ": not found -" << dic->getNdbError() << endl;
-      return NDBT_ProgramExit(NDBT_FAILED);
+      if( using_default_database )
+      {
+        ndbout << "Please specify database name using the -d option. "
+               << "Use option --help for more details." << endl;
+      }
+      else
+      {
+        ndbout << "Table " << _tabname << ": not found - "
+               << dic->getNdbError() << endl;
+      }
+      ndb_free_defaults(argv);
+      return NdbToolsProgramExitCode::FAILED;
     }
   }
   for (int i = 0; _loops == 0 || i < _loops; i++) {
@@ -344,7 +408,9 @@ int main(int argc, char** argv){
   }
   delete ndb;
   delete ndb_cluster_connection;
-  return NDBT_ProgramExit(NDBT_OK);
+
+  ndb_free_defaults(argv);
+  return NdbToolsProgramExitCode::OK;
 }
 
 // vim: set sw=4:
