@@ -290,6 +290,10 @@ struct view {
      "page_requests_direct_return, page_requests_wait_queue, page_requests_wait_io "
      "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>diskpagebuffer`"
   },
+  { "error_messages",
+    "SELECT error_code, error_description, error_status, error_classification "
+    "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>error_messages`"
+  },
   {
     "locks_per_fragment",
     "SELECT name.fq_name, parent_name.fq_name AS parent_fq_name, "
@@ -310,6 +314,8 @@ struct view {
     " CASE log_type"
     "  WHEN 0 THEN \"REDO\""
     "  WHEN 1 THEN \"DD-UNDO\""
+    "  WHEN 2 THEN \"BACKUP-DATA\""
+    "  WHEN 3 THEN \"BACKUP-LOG\""
     "  ELSE \"<unknown>\" "
     " END AS log_type, "
     "log_id, log_part, total, used "
@@ -380,9 +386,7 @@ struct view {
     "  SUM(total*entry_size) AS total,"
     "  SUM(total) AS total_pages "
     "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>pools` "
-    "WHERE ( block_number IN (248, 254) AND "
-    "  (pool_name = \"Index memory\" OR pool_name = \"Data memory\") "
-    ") OR pool_name = \"Long message buffer\" "
+    "WHERE block_number = 254 "
     "GROUP BY node_id, memory_type"
   },
   { "nodes",
@@ -468,7 +472,7 @@ struct view {
     "SELECT node_id, "
     " CASE resource_id"
     "  WHEN 0 THEN \"RESERVED\""
-    "  WHEN 1 THEN \"DISK_OPERATIONS\""
+    "  WHEN 1 THEN \"TRANSACTION_MEMORY\""
     "  WHEN 2 THEN \"DISK_RECORDS\""
     "  WHEN 3 THEN \"DATA_MEMORY\""
     "  WHEN 4 THEN \"JOBBUFFER\""
@@ -479,7 +483,7 @@ struct view {
     "  WHEN 9 THEN \"SCHEMA_TRANS_MEMORY\""
     "  ELSE \"<unknown>\" "
     " END AS resource_name, "
-    "reserved, used, max "
+    "reserved, used, max, spare "
     "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>resources`"
   },
   {"restart_info",
@@ -674,6 +678,12 @@ struct lookup {
   { "<TABLE_PREFIX>dict_obj_types",
     "type_id INT UNSIGNED, "
     "type_name VARCHAR(512)",
+  },
+  { "<TABLE_PREFIX>error_messages",
+    "error_code INT UNSIGNED, "
+    "error_description VARCHAR(512), "
+    "error_status VARCHAR(512), "
+    "error_classification VARCHAR(512)",
   }
 };
 
@@ -792,7 +802,7 @@ int main(int argc, char** argv){
       "config_params",
       "dict_obj_types",
       "ndb$dblqh_tcconnect_state",
-      "ndb$dbtc_apiconnect_state",
+      "ndb$dbtc_apiconnect_state"
     };
     printf("# Drop obsolete lookups in %s\n", opt_ndbinfo_db);
     for (size_t i = 0; i < sizeof(old_lookups)/sizeof(old_lookups[0]); i++)
@@ -884,6 +894,8 @@ int main(int argc, char** argv){
   delete[] tables;
   tables = NULL;
 
+  delete [] tables;
+
   printf("# Recreate handler local lookup tables in %s\n", opt_ndbinfo_db);
   const char* prev_name = "";
   for (size_t i = 0; i < num_lookups; i++)
